from __future__ import print_function
import numpy as np

from asap3 import FullNeighborList
from asap3.MonteCarlo.Moves.Data import lattice as latticedata
from asap3.MonteCarlo.Moves.Base import Move, RandChoice
from ase.cluster.cluster import Cluster

class SurfaceMove(Move):
    def __init__(self, debug=0):
        self.debug = debug
        self.surface_i = -1
        self.vacant_i = -1
        Move.__init__(self)

    def set_atoms(self, atoms):
        Move.set_atoms(self, atoms)

        if not isinstance(atoms, Cluster):
            raise Warning('The cluster is not a valid Cluster instance.')

        if atoms.symmetry != 'fcc':
            raise Warning('Can only work with fcc structures.')

        self.natoms = len(atoms)

        # Get the structual parameters
        sym = atoms.symmetry
        self.neighbor_positions = (latticedata[sym]['neighbor_positions'] *
                                   atoms.lattice_basis[0, 0])
        self.neighbor_cutoff = (latticedata[sym]['neighbor_cutoff'] *
                                atoms.lattice_basis[0, 0])
        self.neighbor_numbers = latticedata[sym]['neighbor_numbers']
        self.neighbor_mapping = latticedata[sym]['neighbor_mapping']
        self.neighbor_count = latticedata[sym]['neighbor_count']
        self.type_count = latticedata[sym]['type_count']
        self.type_names = latticedata[sym]['type_names']
        self.type_numbers = latticedata[sym]['type_numbers']
        self.type_data = latticedata[sym]['type_data']

        # Set neighbors, coordination and type for all atoms
        get_neighbors = FullNeighborList(self.neighbor_cutoff, atoms)

        positions = atoms.get_positions()
        neighbors = []
        coordinations = []
        types = []

        for i, pos in enumerate(positions):
            nl = get_neighbors[i]
            dl = (positions[nl] - pos) / atoms.lattice_basis[0, 0]

            neighbors.append([-1] * self.neighbor_count)
            for n, d in zip(nl, dl):
                name = tuple(d.round(1))
                if name in self.neighbor_numbers:
                    neighbors[i][self.neighbor_numbers[name]] = n

            coordinations.append(self.get_atom_coordination(neighbors[i]))
            types.append(self.get_atom_type(neighbors[i]))

        self.positions = positions
        self.neighbors = np.array(neighbors, dtype=int)
        self.coordinations = np.array(coordinations, dtype=int)
        self.types = np.array(types, dtype=int)

        # Generate vacancies (position, neighbors, coordination and type)
        self.vacant_positions = np.zeros((0, 3), dtype=float)
        self.vacant_neighbors = np.zeros((0, self.neighbor_count), dtype=int)
        self.vacant_coordinations = np.zeros(0, dtype=int)
        self.vacant_types = np.zeros(0, dtype=int)

        for i in self.surface_indexes():
            self.add_vacancies(self.positions[i], self.neighbors[i], i)

        if self.debug > 2:
            if self.check_vacancies():
                raise Warning('Something is wrong, read the message above')

    def __call__(self, atoms):
        self.chose()
        self.backup(self.surface_i)
        atoms[self.surface_i].position = self.vacant_positions[self.vacant_i].copy()

    def chose(self):
        # Choose a surface atom to move and where to move it
        surface_i = RandChoice(self.surface_indexes())

        self_neighbor = True
        while self_neighbor:
            vacant_i = RandChoice(self.vacant_indexes())
            # The atom must not move to a site where its only neighbor is it self
            # If just one other neighbor is found then its fine => break
            for n in self.vacant_neighbors[vacant_i]:
                if n != -1 and n != surface_i:
                    self_neighbor = False
                    break

        self.surface_i = surface_i
        self.vacant_i = vacant_i

    def accept(self):
        # Save the old position, neighbors, type and coordination
        old_position = self.positions[self.surface_i].copy()
        old_neighbors = self.neighbors[self.surface_i].copy()
        old_coordination = self.coordinations[self.surface_i].copy()
        old_type = self.types[self.surface_i].copy()

        # Set the position, neighbors, type and coordination
        new_position = self.vacant_positions[self.vacant_i].copy()
        new_neighbors = self.vacant_neighbors[self.vacant_i].copy()
        new_coordination = self.vacant_coordinations[self.vacant_i].copy()
        new_type = self.vacant_types[self.vacant_i].copy()

        self.positions[self.surface_i] = new_position
        self.neighbors[self.surface_i] = new_neighbors
        self.coordinations[self.surface_i] = new_coordination
        self.types[self.surface_i] = new_type

        # Add "surface_i" in the new neighbors neighborlists
        for j, n in enumerate(new_neighbors):
            if n >= 0 and n != self.surface_i:
                self.neighbors[n][self.neighbor_mapping[j]] = self.surface_i
                self.coordinations[n] += 1
                self.types[n] = self.get_atom_type(self.neighbors[n])
            elif n == self.surface_i:
                self.neighbors[n][j] = -1
                self.coordinations[n] -= 1
                self.types[n] = self.get_atom_type(self.neighbors[n])

        # Set "-1" in the old neighbors neighborlists
        for j, n in enumerate(old_neighbors):
            if n >= 0:
                self.neighbors[n][self.neighbor_mapping[j]] = -1
                self.coordinations[n] -= 1
                self.types[n] = self.get_atom_type(self.neighbors[n])

        # Add the old and remove the new position
        self.remove_vacancy(self.vacant_i)
        self.add_vacancy(old_position,
                         old_neighbors,
                         old_coordination,
                         old_type)

        # Update vacancies around the moved atom old and new position
        self.remove_vacancies(self.surface_i)
        self.add_vacancies(self.positions[self.surface_i],
                           self.neighbors[self.surface_i],
                           self.surface_i)

        #Print that an atom is moved
        if self.debug:
            print (">> Moving atom %i (%2.2f %2.2f %2.2f) " %
                   (self.surface_i, old_position[0],
                    old_position[1], old_position[2]) +
                   "to vacancy %i (%2.2f %2.2f %2.2f):" %
                   (self.vacant_i, new_position[0],
                    new_position[1], new_position[2]))

        if self.debug > 2:
            check_vacancies = self.check_vacancies()
            check_neighbors = self.check_neighbors()

            if check_vacancies or check_neighbors:
                raise Warning('Something is wrong, read messages above')

    # Manipulation functions for vacancies
    def add_vacancy(self, position, neighbors, coordination, type):
        """Adds a single vacancy, without updating any lists."""
        if self.debug:
            print ("Adding vacancy %i (%2.2f %2.2f %2.2f)" %
                   (len(self.vacant_positions), position[0], position[1], position[2]))

        self.vacant_positions = np.append(self.vacant_positions,
                                          position.reshape(1, 3),
                                          axis=0)
        self.vacant_neighbors = np.append(self.vacant_neighbors,
                                          neighbors.reshape(1, self.neighbor_count),
                                          axis=0)
        self.vacant_coordinations = np.append(self.vacant_coordinations, 
                                              coordination)
        self.vacant_types = np.append(self.vacant_types, 
                                      type)

    def remove_vacancy(self, index):
        """Removes a single vacancy, without updating any lists."""
        if self.debug:
            position = self.vacant_positions[index]
            print ("Removing vacacy %i (%2.2f %2.2f %2.2f)" %
                   (index, position[0], position[1], position[2]))

        self.vacant_positions = np.delete(self.vacant_positions, index, axis=0)
        self.vacant_neighbors = np.delete(self.vacant_neighbors, index, axis=0)
        self.vacant_coordinations = np.delete(self.vacant_coordinations, index)
        self.vacant_types = np.delete(self.vacant_types, index)

    def add_vacancies(self, position, neighbors, index):
        """Adds and updates vacancies associated with the atom 
        at "position" indexed with "index". It will only add a
        vacancy if it is not pressent in the list of vacancies
        already."""

        for i, n in enumerate(neighbors):
            if n < 0:
                vacant_position = position + self.neighbor_positions[i] 
                vacant_append = True

                for j, vp in enumerate(self.vacant_positions):
                    if (np.abs(vp - vacant_position) < 1e-3).all():
                        self.vacant_neighbors[j][self.neighbor_mapping[i]] = index
                        self.vacant_coordinations[j] += 1
                        self.vacant_types[j] = self.get_atom_type(self.vacant_neighbors[j])
                        vacant_append = False

                if vacant_append:
                    if self.debug:
                        print ("Adding vacancy %i (%2.2f %2.2f %2.2f)" %
                               (len(self.vacant_positions),
                                vacant_position[0], 
                                vacant_position[1], 
                                vacant_position[2]))

                    self.vacant_positions = np.append(self.vacant_positions,
                                                      vacant_position.reshape(1,3),
                                                      axis=0)
                    self.vacant_neighbors = np.append(self.vacant_neighbors,
                                                      -np.ones((1,self.neighbor_count), dtype=int),
                                                      axis=0)
                    self.vacant_neighbors[-1][self.neighbor_mapping[i]] = index
                    self.vacant_coordinations = np.append(self.vacant_coordinations, 1)
                    self.vacant_types = np.append(self.vacant_types, 
                                             self.get_atom_type(self.vacant_neighbors[-1]))

    def remove_vacancies(self, index):
        """Removes and updates vacancies associated with the 
        atom indexed "index". It will only remove vacancies
        if they are not associated with other atoms."""

        i = 0
        end = len(self.vacant_neighbors)
        while i < end:
            vacant_remove = True

            for j, n in enumerate(self.vacant_neighbors[i]):
                if n >= 0 and n != index:
                    vacant_remove = False
                if n == index:
                    self.vacant_neighbors[i][j] = -1
                    self.vacant_coordinations[i] -= 1
                    self.vacant_types[i] = self.get_atom_type(self.vacant_neighbors[i])

            if vacant_remove:
                if self.debug:
                    position = self.vacant_positions[i]
                    print ("Removing vacacy %i (%2.2f %2.2f %2.2f)" %
                           (i, position[0], position[1], position[2]))

                end -= 1
                self.vacant_positions = np.delete(self.vacant_positions, i, axis=0)
                self.vacant_neighbors = np.delete(self.vacant_neighbors, i, axis=0)
                self.vacant_coordinations = np.delete(self.vacant_coordinations, i)
                self.vacant_types = np.delete(self.vacant_types, i)
            else:
                i += 1

    # Some useful functions
    def surface_indexes(self):
        """Returns the indexes for all the surface atoms (coordination < 12)."""
        mask = self.coordinations < 12
        return np.arange(self.natoms)[mask]

    def vacant_indexes(self):
        """Returns the indexes for the vacancies."""
        return np.arange(len(self.vacant_positions))

    def get_atom_coordination(self, neighbors):
        neighbors = np.array(neighbors)
        neighbors = neighbors[neighbors >= 0]
        return len(neighbors)

    def get_atom_type(self, neighbors):
        neighbors = np.array(neighbors)
        typename = tuple((neighbors >= 0).astype(int))
        if typename in self.type_names:
            return self.type_data[self.type_numbers[typename]]['type']
        else:
            return 0

    # Functions to check the neighbor lists and vacancies
    def check_neighbors(self):
        print(">> Checking the neighbor list...")
        positions = self.positions
        neighbors = self.neighbors
        coordinations = self.coordinations
        types = self.types
        trouble = False

        for i, n in enumerate(neighbors):
            for j, m in enumerate(n):
                found = False
                p1 = positions[i] + self.neighbor_positions[j]

                for k, p2 in enumerate(positions):
                    if (np.abs(p1 - p2) < 1e-3).all():
                        found = True
                        if m < 0:
                            print('Atom %i is missing neighbor atom %i at %s' % (i, k, p2))
                            trouble = True
                        elif m != k:
                            print ('Atom %i is having atom %i as ' +
                                   'neighbor instead of atom %i' % (i, m, k))
                            trouble = True

                if m >= 0 and found is False:
                    trouble = True
                    print('Atom %i does not have atom %i as neighbor' % (i, m))

            if self.get_atom_coordination(n) != coordinations[i]:
                trouble = True
                print('Atom %i coordination does not match' % i)

            if self.get_atom_type(n) != types[i]:
                trouble = True
                print('Atom %i type does not match' % i)

        return trouble

    def check_vacancies(self):
        print(">> Checking the vacancies...")
        vacant_positions = self.vacant_positions
        vacant_neighbors = self.vacant_neighbors
        positions = self.positions
        trouble = False
        
        for i, p1 in enumerate(vacant_positions):
            for j, p2 in enumerate(positions):
                if (np.abs(p1 - p2) < 1e-3).all():
                    trouble = True
                    print("Vacancy %i is identical to %i at %s" % (i, j, p1))

        for i, n in enumerate(vacant_neighbors):
            for j, m in enumerate(n):
                p1 = vacant_positions[i] + self.neighbor_positions[j]

                for k, p2 in enumerate(positions):
                    if (np.abs(p1 - p2) < 1e-3).all():
                        if m < 0:
                            print('Vacancy %i is missing neighbor atom %i at %s' % (i, k, p2))
                            trouble = True
                        elif m != k:
                            print ('Vacancy %i is having atom %i as ' % (i, m) +
                                   'neighbor instead of atom %i' % k)
                            trouble = True

        return trouble




from __future__ import print_function
from ase.units import kB, fs
from asap3 import EMT
from ase.build import fcc111
from ase.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.langevin import Langevin
from asap3.constraints import FixAtoms
from asap3.testtools import ReportTest
import numpy as np

def print_momenta(atoms):
    p = atoms.get_momenta()
    p1 = p[:len(atoms)//2]
    p2 = p[len(atoms)//2:]
    print("Momenta:", (p1*p1).sum(), (p2*p2).sum())

T = 4
print("Expecting temperature around {} K\n".format(T))

atoms = fcc111('Au', size=(4,4,4),vacuum=10.0)
atoms.set_calculator(EMT())

n_atoms = atoms.get_number_of_atoms()
c = FixAtoms(indices=range(n_atoms//2))
atoms.set_constraint(c)

print_momenta(atoms)
MaxwellBoltzmannDistribution(atoms, 300*kB)
print_momenta(atoms)
dyn = Langevin(atoms, timestep=0.5*fs, temperature=T*kB, friction=1e-1)
total_run = (2000, 10000)
dyn.attach(lambda: print(atoms.get_temperature()), interval=100)
dyn.run(total_run[0])

print("Computing average temperature.")
temp_data = []
dyn.attach(lambda: temp_data.append(atoms.get_temperature()), interval=10)
dyn.run(total_run[1])
Tmean = np.mean(temp_data)
print_momenta(atoms)

print("Average temperature is:", Tmean, "K")
ReportTest("Average temperature", Tmean, T, T/4.0)
ReportTest.Summary()




#!/usr/bin/env python

"""Get Python configuration variables.

Usage: python getconfig.py VARIABLE

Used to extract variables from Python's Makefile, intended to be
called from Asap's Makefile.
"""
from __future__ import print_function

import sys
import distutils.sysconfig

if len(sys.argv) != 2:
    print("\nERROR: Got %s arguments, expected 1.\n\n" % (len(sys.argv)-1,), file=sys.stderr)
    print(__doc__, file=sys.stderr)
    sys.exit(-1)

key = sys.argv[1]
if key == "SITEPACKAGES":
    print(distutils.sysconfig.get_python_lib(plat_specific=True))
else:
    cfgDict = distutils.sysconfig.get_config_vars()
    if key == 'BLDLIBRARY':
        val = cfgDict.get(key, None)
        if not val:
            val = '-lpython{0}'.format(cfgDict['VERSION'])
        print(val)
    else:
        print(cfgDict[key])
        



from __future__ import print_function
from asap3 import *
from numpy import *
from pickle import load, dump
from asap3.testtools import ReportTest
import ase.data
import sys

timeunit = 1.018047e-14             # Seconds
femtosecond = 1e-15 / timeunit      # Marginally different from units.fs

print_version(1)

if sys.version_info[0] >= 3:
    picklehack = {'encoding': 'latin1'}
else:
    picklehack = {}


if hasattr(ase.data, 'atomic_masses_legacy'):
    # Newer ASE
    picklefile = "Verlet.pickle"
else:
    picklefile = "Verlet_legacymass.pickle"

data = load(open(picklefile, "rb"), **picklehack)
init_pos = array(data["initial"])
init_pos.shape = (-1,3)
init_box = array(data["box"])
init_box.shape = (3,3)
atoms = Atoms(positions=init_pos, cell=init_box)
atoms.set_atomic_numbers(47*ones((len(atoms),)))
atoms.set_calculator(EMT())

dyn = VelocityVerlet(atoms, 2 * femtosecond)
dyn.attach(MDLogger(dyn, atoms, '-', peratom=True), interval=5)

etot = None

for i in range(10):
    dyn.run(20)
    epot = atoms.get_potential_energy() / len(atoms)
    ekin = atoms.get_kinetic_energy() / len(atoms)
    if etot is None:
        etot = epot + ekin
    else:
        ReportTest("Energy conservation:", epot + ekin, etot, 0.001)
        
final_pos = array(data["final"])
diff = max(abs(atoms.get_positions().flat - final_pos))
print("Maximal deviation of positions:", diff)
ReportTest("Maximal deviation of positions", diff, 0, 1e-9)

diff = max(abs(atoms.get_stresses().flat - array(data["stress"])))

print("Maximal deviation of stresses:", diff)
ReportTest("Maximal deviation of stresses", diff, 0, 1e-9)
ReportTest.Summary()




from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from ase.lattice.cubic import FaceCenteredCubic
from asap3.setup.multidislocation import random_dislocations
from asap3.setup.dislocation import Dislocation
import ase.data
from .helpers import create_atoms
import numpy as np

def make_dislocated_nanocrystal(size, centres, rotations, 
                                dislocationdensity,
                                symbol, latticeconstant=None,
                                min_dist=2.0,
                                multiproc=False):
    """Create a nanocrystalline sample with pre-existing dislocation density.

    Parameters:
      size:      Size of the system, in Angstrom (either a number or three numbers).
      centres:   Positions of grains in scaled coordinates.
      rotations: Rotation matrices for the grains.
      dislocationdensity:
                 Dislocation density in m^-2
      symbol:    Chemical symbol or atomic number.
      latticeconstant (optional):
                 Overrides the default lattice constant.
      min_dist (optional):
                 If two atoms are closer than this distance, one of them is removed
                 (default: 2.0).
      multiproc: Use all cores to calculate dislocation strain fields (default: False).
    """

    assert type(size) == float or type(size) == list
    if type(size) == float:
        size = np.array([size, size, size])
    elif type(size) == list:
        size = np.array(size)
    assert size.shape == (3,)
    assert len(centres) == len(rotations)
    centres *= size
    if latticeconstant is None:
        # Get lattice constant from the element
        if isinstance(symbol, int):
            z = symbol
        else:
            z = ase.data.atomic_numbers[symbol]
        if ase.data.reference_states[z]['symmetry'] != 'fcc':
            raise ValueError("Cannot extract FCC lattice constant from non-fcc element ({})".format(symbol))
        latticeconstant = ase.data.reference_states[z]['a']
    unit = FaceCenteredCubic(symbol=symbol, size=(1,1,1), pbc=True,
                             latticeconstant=latticeconstant)
    # Create a dislocation field for each grain.  The radius of the dislocation
    # field should be half the sidelength of the overall system
    dislocationfields = []
    for i in range(len(centres)):
        disloc_params = random_dislocations(max(size)/2.0, dislocationdensity)
        n_dislocs = len(disloc_params)
        dislocs = [Dislocation(point, line, latticeconstant * burgers)
                   for point, burgers, line in disloc_params]
        dislocationfields.append(dislocs)

    return create_atoms(centres, rotations, unit, size, min_dist,
                        dislocations=dislocationfields, multiproc=multiproc)




'''
Created on Mar 16, 2011

@author: schiotz
'''

from ase.md.md import MolecularDynamics
from ase.md.verlet import VelocityVerlet as VelocityVerlet_ASE
import ase
import asap3
import numpy as np
import sys

class VelocityVerlet_Asap(MolecularDynamics):
    def __init__(self, atoms, dt, trajectory=None, logfile=None,
                 loginterval=1):
        MolecularDynamics.__init__(self, atoms, dt, trajectory, logfile,
                                   loginterval)
        if not atoms.has('momenta'):
            atoms.set_momenta(np.zeros((len(atoms), 3), float))
        self.calculator = atoms.get_calculator()
        self.asap_md = asap3._asap.VelocityVerlet(atoms, self.calculator, dt)
        # Identify FixAtoms constraint
        if atoms.constraints:
            assert len(atoms.constraints) == 1
            constraint = atoms.constraints[0]
            assert isinstance(constraint, asap3.constraints.FixAtoms)
            constraint.prepare_for_asap(atoms)
            mask = constraint.index
            assert mask.shape == (len(atoms),) and mask.dtype == bool
            mult = np.logical_not(mask).astype(float)
            self.atoms.arrays["FixAtoms_mult_double"] = mult
            
    def run(self, steps):
        assert(self.calculator is self.atoms.get_calculator())
        self.asap_md.run(steps, self.observers, self)
        
def VelocityVerlet(atoms, dt, trajectory=None, logfile=None, loginterval=1):
    if isinstance(atoms, ase.Atoms) and asap3.constraints.check_asap_constraints(atoms):
        sys.stderr.write("Using Asap-optimized Verlet algorithm\n")
        return VelocityVerlet_Asap(atoms, dt, trajectory, logfile, loginterval)
    else:
        sys.stderr.write("Using ASE Verlet algorithm\n")
        return VelocityVerlet_ASE(atoms, dt, trajectory, logfile, loginterval)




"""Test local crystal structure analysis (CNA and Coordination number)."""
from __future__ import print_function

from asap3 import EMT
from asap3.testtools import ReportTest
from ase.optimize import FIRE
from ase.lattice import bulk
import numpy as np

atoms = bulk('Cu')
atoms = atoms.repeat((7, 7, 100))

print("Testing FIRE relaxation")
atoms.set_calculator(EMT())
e0 = atoms.get_potential_energy()
natoms = atoms.get_number_of_atoms()
ReportTest("Number of atoms", natoms, 4900, 0)
print("Potential energy before perturbation:", e0, e0/natoms)

dx = 0.1 * np.sin(42 * np.arange(3 * len(atoms)))
dx.shape = (-1, 3)
atoms.set_positions(dx + atoms.get_positions())
e1 = atoms.get_potential_energy()
print("Potential energy after perturbation:", e1, e1/natoms)

dyn = FIRE(atoms)
dyn.run(0.01)
e2 = atoms.get_potential_energy()
print("Potential energy after perturbation:", e2, e2/natoms)
ReportTest("Energy after relaxation", e2, e0, 0.02)
ReportTest("Number of FIRE steps", dyn.nsteps, 61, 5)


ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import *
from numpy import *
import ase.data

def TestLists(nblist, fnb, name, count=None):
    "Run the tests on a half and a full neighbor list."
    print("")
    if count:
        print("Testing %s: Length of lists" % (name,))
        sum = 0
        for lst in nblist:
            sum += len(lst)
        ReportTest("   Half list", sum, count*len(atoms), 0)

        lfnb = list(map(len, fnb))
        assert len(lfnb) == len(atoms)
        ReportTest("   Shortest full list", min(lfnb), 2*count, 0)
        ReportTest("   Longest full list", max(lfnb), 2*count, 0)

    print ("Testing %s: Symmetry of full list; full list atoms on half-lists."
           % (name,))
    for i, nb in enumerate(fnb):
        for jj in nb:
            j = int(jj)
            ReportTest.BoolTest("Atom %d on list %d" % (j, i), i in fnb[j],
                                silent=True)
            ReportTest.BoolTest("Exactly one of atoms %d and %d on half-lists"
                                % (j, i),
                                (i in nblist[j]) != (j in nblist[i]),
                                silent=True)
        if ReportTest.GetNumberOfErrors() > 10:
            print("*** Too many errors - giving up! ***")
            break

    print("Testing %s: Half-list atoms on full list." % (name,))
    for i, nb in enumerate(nblist):
        for jj in nb:
            j = int(jj)
            ReportTest.BoolTest("Atom %d on list %d (forward)" % (j, i),
                                j in fnb[i], silent=True)
            ReportTest.BoolTest("Atom %d on list %d (reverse)" % (i, j),
                                i in fnb[j], silent=True)
        if ReportTest.GetNumberOfErrors() > 10:
            print("*** Too many errors - giving up! ***")
            break
    

print_version(1)

element = "Cu"
latconst = ase.data.reference_states[ase.data.atomic_numbers[element]]['a']

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,1],[0,0,1]], size=(9,7,5),
                          symbol=element, debug=0)
atoms.set_calculator(EMT(minimum_image=True))
epot = atoms.get_potential_energy()

nblist = atoms.get_calculator().get_neighborlist()
count = {}
for lst in nblist:
    n = len(lst)
    try:
        count[n] += 1
    except KeyError:
        count[n] = 1
# print "Histogram:"
numbers = sorted(count.keys())
sum = 0
for i in numbers:
    #print i, count[i]
    sum += i*count[i]

ReportTest("Number of neighbors (EMT's NB list)", sum, 21*len(atoms), 0)

nblist = NeighborList(latconst * 0.5 * (1/sqrt(2) + 1), atoms, 0.0)
#nblist = NeighborCellLocator(latconst * 0.5 * (1/sqrt(2) + 1), atoms, 0.0)
fnb = FullNeighborList(latconst * 0.5 * (1/sqrt(2) + 1), Atoms(atoms))
TestLists(nblist, fnb, "nearest-neigbor lists (periodic)", 6)

ReportTest("Energy unperturbed 1", atoms.get_potential_energy(), epot, 1e-11)
atoms.set_positions(atoms.get_positions())
ReportTest("Energy unperturbed 2", atoms.get_potential_energy(), epot, 1e-11)

nblist = NeighborList(4.98409, atoms, 0.0)
fnb = FullNeighborList(4.98409, Atoms(atoms))
TestLists(nblist, fnb, "long neigbor lists (periodic)", 21)

ReportTest("Energy unperturbed 3", atoms.get_potential_energy(), epot, 1e-11)
atoms.set_positions(atoms.get_positions())
ReportTest("Energy unperturbed 4", atoms.get_potential_energy(), epot, 1e-11)

atoms = Atoms(atoms, pbc=(0,0,0))

nblist = NeighborList(latconst * 0.5 * (1/sqrt(2) + 1), atoms, 0.0)
fnb = FullNeighborList(latconst * 0.5 * (1/sqrt(2) + 1), Atoms(atoms))
TestLists(nblist, fnb, "nearest-neigbor lists (non-periodic)")

atoms = Atoms(atoms, pbc=(0,1,0))

nblist = NeighborList(4.98409, atoms, 0.0)
fnb = FullNeighborList(4.98409, Atoms(atoms))
TestLists(nblist, fnb, "long neigbor lists (semi-periodic)")

ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from numpy import *
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from ase.lattice.compounds import *

def check(a1, e):
    a2 = a1.copy()
    a1.set_calculator(EMT(EMTStandardParameters()))
    a2.set_calculator(EMT())
    ReportTest("Energy (%s)" % (e,), a2.get_potential_energy(),
               a1.get_potential_energy(), 1e-8)
    f1 = a1.get_forces().ravel()
    f2 = a2.get_forces().ravel()
    ReportTest("Forces (%s)" % (e,), max(fabs(f2-f1)), 0.0, 1e-8)

elements = ["Al", "Ni", "Cu", "Pd", "Ag", "Pt", "Au"]
print("Comparing Default and Standard parameter providers.")
for e in elements:
    atoms = FaceCenteredCubic(size=(4,4,4), symbol=e)
    check(atoms, e)
    
for i in range(1, len(elements)):
    e1 = elements[i]
    e2 = elements[i-1]
    atoms = B2(size=(4,4,4), symbol=(e1,e2), latticeconstant=3.5, pbc=False)
    check(atoms, e1+e2)

    atoms = B2(size=(4,4,4), symbol=(e2,e1), latticeconstant=3.5, pbc=False)
    check(atoms, e2+e1)

ReportTest.Summary()




# Checks for Ticket #11

"Test for correct energy calculations with varying unit cell."
from __future__ import print_function

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

print(__doc__)

for pbc in (True, False, (1,1,0)):
    for scale in (True, False):
        print("Running test with pbc=%s and scale_atoms=%s" % (pbc, scale))
        
        atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                                  size=(6,6,6), symbol="Cu", pbc=pbc)
        atoms.set_calculator(EMT())
        uc = atoms.get_cell()
        atoms.get_potential_energy()
        for factor in (1.0, 1.01, 1.02, 1.1, 1.5, 1.49, 1.4, 1.4, 1.0, 0.9):
            atoms.set_cell(uc * factor, scale_atoms=scale)
            f = atoms.get_forces()
            e = atoms.get_potential_energy()
            atoms2 = Atoms(atoms)
            atoms2.set_calculator(EMT())
            e2 = atoms2.get_potential_energy()
            f2 = atoms2.get_forces()
            name = "(factor = %.3f  PBC = %s  scale_atoms = %s)" % (factor,
                                                                    pbc, scale)
            ReportTest("Energy "+name, e, e2, 1e-6)
            maxf = max(abs(f.flat[:] - f2.flat[:]))
            ReportTest("Forces "+name, maxf, 0.0, 1e-6)

ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from numpy import *
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from ase.lattice.compounds import L1_2
from OpenKIM_modelname import openkimmodel
from ase.data import reference_states, atomic_numbers

element = 'Au'
element2 = 'Ag'
sizes = arange(10,0,-1)

energy = None
force = None
for s in sizes:
    atoms = FaceCenteredCubic(symbol=element, size=(s,s,s), pbc=True)
    r = atoms.get_positions()
    for i in range(0, len(r), 4):
        r[i][0] += 0.1
    atoms.set_positions(r)
    atoms.set_calculator(EMT())
    e = atoms.get_potential_energy() / len(atoms)   
    print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
    if energy is None:
        energy = e
    else:
        ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                   e, energy, 1e-8)
    f = atoms.get_forces()
    if force is None:
        force = f[:4]
    else:
        for i in range(len(f)):
            for j in range(3):
                ReportTest("Force for size %i atom %i of %i component %i"
                           % (s, i, len(atoms), j),
                           f[i,j], force[i % 4, j], 1e-8, silent=True)
        #print force
        #print f[:10]
    del atoms

# Lattice constant for alloy
lc = [reference_states[atomic_numbers[s]]['a'] for s in (element, element2)]
lc = (3*lc[0] + lc[1])/4.0

energy = None
force = None
for s in sizes:
    atoms = L1_2(symbol=(element,element2), size=(s,s,s), latticeconstant=lc, pbc=True)
    r = atoms.get_positions()
    for i in range(1, len(r), 4):
        r[i][0] += 0.1
    atoms.set_positions(r)
    atoms.set_calculator(EMT())
    e = atoms.get_potential_energy() / len(atoms)   
    print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
    if energy is None:
        energy = e
    else:
        ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                   e, energy, 1e-8)
    f = atoms.get_forces()
    if force is None:
        force = f[:4]
    else:
        for i in range(len(f)):
            for j in range(3):
                ReportTest("Force for size %i atom %i of %i component %i"
                           % (s, i, len(atoms), j),
                           f[i,j], force[i % 4, j], 1e-8, silent=True)
        #print force
        #print f[:10]
    del atoms


ReportTest.Summary()



import numpy as np
from asap3.nanoparticle_mc import data
from ase.atom import Atom, names as oldnames
from ase.data import atomic_numbers, chemical_symbols, reference_states
from asap3.Internal.MonteCarloAtoms import MC_Atom
class ClusterAtom(MC_Atom):
    """Cluster Atom"""
    names = oldnames.copy()
    names['neighbors'] = ('neighbors', None)
    names['coordination'] = ('coordinations', 0)
    names['type'] = ('types', 0)

    _data = []

    def __init__(self, symbol='X', position=(0.0, 0.0, 0.0), atoms=None, index=None):
        self.atoms = atoms
        self.index = index

        if atoms is None:
            if isinstance(symbol, str):
                self.number = atomic_numbers[symbol]
            else:
                self.number = symbol
 
            self.position = np.array(position, float)

    def __repr__(self):
        output = 'ClusterAtom(%s, %s' % (self.symbol, self.position.tolist())

        for name in self._data:
            if name != 'number' and name != 'position' and self._get(name) is not None:
                output += ', %s=%s' % (name, self._get(name))

        return output + ')'

    def get_raw(self, name):
        """Get attribute, return None if not explicitely set."""
        if name == 'symbol':
            return chemical_symbols[self.get_raw('number')]

        if self.atoms is None:
            return self.data[name]
        
        plural = self.names[name][0]
        if plural in self.atoms.arrays:
            return self.atoms.arrays[plural][self.index]
        else:
            return None

    def get(self, name):
        """Get attribute, return default if not explicitely set."""
        value = self.get_raw(name)
        if value is None:
            if name == 'mass':
                value = atomic_masses[self.number]
            else:
                value = self.names[name][1]
        return value



    #def _get_copy(self, name):
    #    return self._get(name, copy=True)

                
    def set(self, name, value, copy=False):
        """Set attribute."""
        if name == 'symbol':
            name = 'number'
            value = atomic_numbers[value]

        if self.atoms is None or copy:
            assert name in self.names
            self.data[name] = value
        else:
            plural, default = self.names[name]
            if plural in self.atoms.arrays:
                array = self.atoms.arrays[plural]
                if name == 'magmom' and array.ndim == 2:
                    assert len(value) == 3
                array[self.index] = value
            else:
                if name == 'magmom' and np.asarray(value).ndim == 1:
                    array = np.zeros((len(self.atoms), 3))
                elif name == 'mass':
                    array = self.atoms.get_masses()
                else:
                    default = np.asarray(default)
                    array = np.zeros((len(self.atoms),) + default.shape,
                                     default.dtype)
                array[self.index] = value
                self.atoms.new_array(plural, array)


    def has(self, name):
        return name in self._data

    def cut_reference_to_atoms(self):
        for name, a in self.atoms.arrays.items():
            self.set(self.atoms.names[name][0], a[self.index].copy(), True)
        self.atoms = None
        self.index = None

    def get_symbol(self): return chemical_symbols[self.get('number')]
    def get_neighbors(self): return self.get('neighbors')
    def get_type(self): return self.get('type')
    def get_coordination(self): return self.get('coordination')
    #def get_(self): return self._get('')

    def set_symbol(self, value): self.set('number', atomic_numbers[value])
    def set_neighbors(self, value): self.set('neighbors', np.array(value, int))
    def set_type(self, value): self.set('type', value)
    def set_coordination(self, value): self.set('coordination', value)
    #def get_(self, value): return self._set('', value)

    symbol = property(get_symbol, set_symbol, doc='Chemical symbol')
    neighbors = property(get_neighbors, set_neighbors, doc='List of nearest neighbors')
    type = property(get_type, set_type, doc='Atom type')
    coordination = property(get_coordination, set_coordination, doc='Atom coordination')
    # We need to repeat these as MC_Atom has blocked them.
    # Lambda expression needed as _get_position is in a base class :-(
    position = property(lambda self: self._get_position(),
                        lambda self, x : self.set_position(x),
                        doc='XYZ-coordinates')
    number = property(lambda self: self.get_atomic_number(),
                      lambda self, x: self.set_atomic_number(x),
                      doc='Atomic number')



#!/usr/bin/python
# Copyright (C) 2007  CAMP
# Please see the accompanying LICENSE file for further information.

# Tests the LennardJones potential.

#import ASE.ChemicalElements.mass
from __future__ import print_function
from ase import Atoms, data

from asap3 import *
from ase.lattice.cubic import *
from asap3.testtools import ReportTest
from asap3.md.verlet import VelocityVerlet

from StressModule import *
from time import time, localtime, strftime
import numpy as np

print_version(1)
#AsapThreads()

Silent=True

# Calculates Emin radius:
# 4*epsilon*((sigma/rij)^12-(sigma/rij)^6)
# f(r)  = 4*e*((s/r)^(12)-(s/r)^(6))
# f'(r) = 4*e*(6*s^6/r^7-12*s^12/r^13)
#         4*e*(6*s^6/r^7-12*s^12/r^13)=0
#         r = s*2^(1/6)
def EminRadius(sigma):
 return sigma*1.122462048;

#It is expected that E=0 because r>rcut
def twoAtomTest():
  print("\nRunning LJ twoAtomTest")
  #print ''
  elements = [29]
  epsilon  = [0.15]
  sigma    = [2.7]

  atoms=Atoms(positions=[(0.0, 0.0, 0.0),(10.0, 10.0, 10.0)],
              symbols="2Cu")
  atoms.center(vacuum=1.0)

  atoms.set_calculator(LennardJones(elements, epsilon, sigma, -1.0, False))

 
  result = atoms.get_potential_energy()
  #print "Energy is ", result;
  ReportTest("Expected energy for two atoms with r>rcut", result, 0, 1e-12, silent=Silent)

#it is expected that E=-epsilon because we calculate r that way
def twoAtomClose():
  print("\nRunning LJ twoAtomClose")
  elements = [29]
  epsilon  = [0.15]
  sigma    = [2.7]

  EminR = EminRadius(sigma[0])

  atoms=Atoms(positions=[(0.0, 0.0, 0.0),(0.0, 0.0, EminR)],
                    symbols="Cu2")
  atoms.center(vacuum=1.0)

  atoms.set_calculator(LennardJones(elements, epsilon, sigma, -1.0, False))


  result = atoms.get_potential_energy()
  ReportTest("  Expected energy for two atoms with r->Emin=-epsilon", result, -epsilon[0], 1e-14, silent=Silent)

#two atoms
def twoAtomForce():
  print("\nRunning LJ twoAtomForce")
  elements = [29]
  epsilon  = [0.15]
  sigma    = [2.7]

  atoms=Atoms(positions=[(0.0, 0.0, 0.0),(0.0, 0.0, 2.0)],
              symbols="Cu2")
  atoms.center(vacuum=1.0)

  atoms.set_calculator(LennardJones(elements, epsilon, sigma, -1.0, False))


  result = atoms.get_potential_energy()

  r = atoms.get_positions()
  r.flat[:] += 0.06 * np.sin(np.arange(3*len(atoms)))
  atoms.set_positions(r)
  resA, resB = atoms.get_forces() ;
  storedForces = [ -1.15732425,   13.10743025, -258.04517252 ]

  for a in range(1,3):
    ReportTest(("  Simple force test dimension %d" % a), resA[a],
               storedForces[a], 1e-8, silent=Silent)


  print("  Running Verlet dynamics (%s)" % ("Cu",))
  dyn = VelocityVerlet(atoms, 2*units.fs)
  dyn.run(100)

  etot1 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
  dyn.run(1000)
  etot2 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
  ReportTest(("  Energy conservation (%s)" % ("Cu",)), etot1, etot2, 1.0, silent=Silent)

  epot = atoms.get_potential_energies()
  stress = atoms.get_stresses()

  e = []
  s = []
  j = 0
  for i in range(0, len(atoms), 100):
      e.append(epot[i])
      s.append(stress[i,j])
      j = (j + 1) % 6
  print("  e"+"Cu"+" =", repr(e))
  print("  s"+"Cu"+" =", repr(s))

      
#
# size=15:     will allocate the size of the cube to size=(15,15,15)
# before=100:  will run the simulation for 100*2*femtosecond before starting measuring
# steps=200:   will measure and compare the potentials for 200*(2*femtosecond)
# v0:          True: add v0 False: do not add v0
# rCut:        -1: automatically set; or any value >0
# nonSense:     uses a random set of values to test whether multidimensional params work
def dynamicsTest(size, before, steps, v0, rCut, nonSense):
    print("\n\nStarting LJ dynamicsTest with ")
    print("  cube of size                  %d"%size)
    print("  time before measurement start %d*1*femtoseconds"%before)
    print("  measuring time                %d*femtoseconds"%steps)
    print("  add v0                        %d"%(v0==1))
    print("  rCut                          %d"%rCut)
    print("  3 kinds of atoms              %s"%(nonSense==1))

    start = time();

    nsteps = steps;
    #Warning: numbers and constants are not supposed to make sense or to be correct!!
    #         they're only here to assure that the program handles the input correctly
    if nonSense:
      elements = [29, 79, 39]
      epsilon  = [0.15, 0.00, 0.00,
                  0.10, 0.25, 0.00,
                  0.31, 0.60, 0.15]
      sigma    = [2.7 , 0.00, 0.00,
                  1.90, 2.7 , 0.00,
                  2.20, 1.50, 2.7 ]
      atoms = L1_2([[1,0,0],[0,1,0],[0,0,1]], size=(size,size,size),
                              element=("Cu", "Au"), periodic=(1,0,1), debug=0,
                              latticeconstant=3.95)
      atoms.set_calculator(LennardJones(elements, epsilon, sigma, rCut, v0))
    else:
      elements = [29]
      epsilon  = [0.15]
      sigma    = [2.7]
      atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                                size=(size,size,size),
                                symbol="Cu", pbc=(1,0,1), debug=0,
                                latticeconstant=1.09*sigma[0]*1.41)
      atoms.set_calculator(LennardJones(elements, epsilon, sigma, rCut, v0))

    r = atoms.get_positions()
    r.flat[:] += 0.06 * np.sin(np.arange(3*len(atoms)))
    atoms.set_positions(r)

    print("  Running Verlet dynamics (%s)" % ("Cu",))
    dyn = VelocityVerlet(atoms, 1*units.fs)
    dyn.run(before)
    
    etot1 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
    for i in range(nsteps//10):
        dyn.run(10)
        #print atoms.get_potential_energy()/len(atoms), atoms.get_kinetic_energy()/len(atoms), (atoms.get_potential_energy() + atoms.get_kinetic_energy())/len(atoms)
    etot2 = (atoms.get_potential_energy() + atoms.get_kinetic_energy())
    ReportTest(("  Energy conservation (%s)" % ("Cu",)), etot1, etot2, 2, silent=Silent)
    print("Before: ", etot1, " now: ", etot2, " diff= ", (etot1-etot2))
    epot = atoms.get_potential_energies()
    stress = atoms.get_stresses()
    print("  Reporting energies and stresses")
    
    e = []
    s = []
    j = 0
    for i in range(0, len(atoms), 100):
        e.append(epot[i])
        s.append(stress[i,j])
        j = (j + 1) % 6
    print("  e"+"Cu"+" =", repr(e))
    print("  s"+"Cu"+" =", repr(s))
    end = time();
    print("  ++++ Test runtime of the LJ dynamicsTest was %d seconds ++++\n\n" %(end-start))
    if 1:
        sumstress = np.zeros(6, np.float)
        for s in stress:
            sumstress += s
        totalstress = atoms.get_stress()
        for i in range(6):
            ReportTest("Sum of stresses (%d)" % (i,), sumstress[i]/len(atoms),
                       totalstress[i], abs(0.01*totalstress[i]))
    

#TODO: create a better test
#some stress tests fail because the model doesn't quite fit:
#    Bulk modulus (Copper_LJ, energies): FAILED, got 85.19709 expected 133.52 (max deviation: 0.1)
#    Bulk modulus (Copper_LJ, pressure): FAILED, got 86.033135 expected 85.19709 (max deviation: 0.5)
#    C11 from stress: FAILED, got 119.95725 expected 171.48 (max deviation: 0.1)
#    C12 from stress: FAILED, got 66.826111 expected 114.71 (max deviation: 0.1)
#    B from C11 and C12: FAILED, got 84.53649 expected 133.52 (max deviation: 0.5)
#    C44 from energies: FAILED, got 66.559653 expected 88.45 (max deviation: 0.1)
#    C44(alt) from energies: FAILED, got 70.219801 expected 66.559653 (max deviation: 0.1)
#    C44(alt) from stresses: FAILED, got 70.24277 expected 66.529593 (max deviation: 0.1)
def stressTest(slow, v0=False):

    start = time();

    elements = [29]
    epsilon  = [0.15]
    sigma    = [2.7]
    if slow:
        alpha = 12.13188
        beta = 14.45392
    else:
        alpha = beta = 12
    fact = (2*alpha/beta)**(1.0/6)
    latconst = fact*sigma[0]*np.sqrt(2.0)
    bookbulk = (2*np.sqrt(2)/9) * (156*alpha/fact**14 - 42*beta/fact**8) * epsilon[0] / (fact * sigma[0]**3) / units.GPa
    
    print("Lattice const. ", latconst)

    if slow:
        symb="Cu_LJ_Full"
        size=12
        rCut=8*sigma[0]
        bookbulk *= 0.9706  # Not fully converged with this rCut
        bookc11 = 123.70
        bookc12 = 69.78
        bookc44 = 69.28
    else:
        symb="Cu_LJ_NN"
        size=10
        rCut = fact * sigma[0] * 0.5 * (1 + np.sqrt(2))
        bookc11 = 85.65
        bookc12 = 41.76
        bookc44 = 41.82

    book[symb] = {"bookbulk": (134.3, bookbulk),
                  "bookc11": (168.3, bookc11),
                  "bookc12": (122.1, bookc12),
                  "bookc44": (75.7, bookc44)}

    atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                              size=(size,size,size),
                              symbol="Cu", pbc=(1,0,1), debug=0,
                              latticeconstant=latconst)

    atoms.set_calculator(LennardJones(elements, epsilon, sigma, rCut, v0))

    atoms.set_momenta(np.zeros((len(atoms),3), np.float))
    #findlatticeconst(atoms, latconst)  ## Lattice const already OK
    elasticconstants(atoms, symb, fitfact=20.0, fitfact2=20.0, **book[symb])
    end = time();
    print("&&&& Test runtime of the LJ stressTest was %d seconds &&&&" %(end-start))


start = time();
twoAtomClose()
twoAtomTest()
twoAtomForce()
#dynamicsTest(10, 10, 200, False, -1, True, False) #3 kinds of atoms, no map
#dynamicsTest(10, 10, 200, False, -1, True, True)  #3 kinds of atoms, map
dynamicsTest(10, 10, 200, False, -1, False) #1 kind of atom
#dynamicsTest(20, 10, 200, True, -1, False)
#dynamicsTest(10, 10, 10000, True, -1, False, False)
#dynamicsTest(10, 10, 10000, True, -1, False, True)
#dynamicsTest(10, 40000, 200, False, -1, False))
stressTest(False)
stressTest(False, True)
#stressTest(True)

end = time();
ReportTest.Summary()
print("&&&& The time for all LJ tests was %d seconds &&&&" %(end-start))



"""Lattice data - Face Centered Cubic"""
from __future__ import print_function
from __future__ import absolute_import

import numpy as np
from math import sqrt
from .symmetry import *

#Definition of symmetries
basesymmetries = [np.matrix([[-1, 0, 0],  #Mirror x-axis
                             [0, 1, 0],
                             [0, 0, 1]]),
                  np.matrix([[1, 0, 0],   #Mirror y-axis
                             [0, -1, 0],
                             [0, 0, 1]]),
                  np.matrix([[1, 0, 0],   #Mirror z-axis
                             [0, 1, 0],
                             [0, 0, -1]]),
                  np.matrix([[1, 0, 0],   #Rotation x-axis (4-fold)
                             [0, 0, -1],
                             [0, 1, 0]]),
                  np.matrix([[0, 0, -1],  #Rotation y-axis (4-fold)
                             [0, 1, 0],
                             [1, 0, 0]]),
                  np.matrix([[0, 1, 0],   #Rotation z-axis (4-fold)
                             [-1, 0, 0],
                             [0, 0, 1]]),
                  np.matrix([[0, 0, 1],   #Rotation (111)-axis (3-fold)
                             [1, 0, 0],
                             [0, 1, 0]]),
                  np.matrix([[0, 0, -1],  #Rotation (11-1)-axis (3-fold)
                             [1, 0, 0],
                             [0, -1, 0]]),
                  np.matrix([[0, 0, 1],   #Rotation (1-11)-axis (3-fold)
                             [-1, 0, 0],
                             [0, -1, 0]]),
                  np.matrix([[0, 0, -1],  #Rotation (-111)-axis (3-fold)
                             [-1, 0, 0],
                             [0, 1, 0]])]

symmetries = get_all_symmetries(basesymmetries, 48)

#Definition of used surfaces
surface_names = [(1,0,0), (-1,0,0),
                 (0,1,0), (0,-1,0),
                 (0,0,1), (0,0,-1), 
                 (1,1,0), (-1,-1,0),
                 (1,0,1), (-1,0,-1), 
                 (0,1,1), (0,-1,-1), 
                 (1,-1,0), (-1,1,0), 
                 (1,0,-1), (-1,0,1),
                 (0,1,-1), (0,-1,1),
                 (1,1,1), (-1,-1,-1),
                 (-1,1,1), (1,-1,-1),
                 (1,-1,1), (-1,1,-1),
                 (1,1,-1), (-1,-1,1)]

surface_numbers = {}
for i, s in enumerate(surface_names):
    surface_numbers[s] = i

surface_count = len(surface_names)

surface_mapping = {0: 1, 1: 0, 2: 3, 3: 2, 4: 5, 5: 4, 
                   6: 7, 7: 6, 8: 9, 9: 8, 10: 11, 11: 10,
                   12: 13, 13: 12, 14: 15, 15: 14, 16: 17, 17: 16,
                   18: 19, 19: 18, 20: 21, 21: 20, 22: 23, 23: 22,
                   24: 25, 25: 24}

surface_data = ([{'l': 1.0, 'd': 0.5}] * 6 +
                [{'l': 1.5, 'd': 0.5 / sqrt(2)}] * 12 +
                [{'l': 1.0, 'd': 1.0 / sqrt(3)}] * 8)

surface_symmetries = get_surface_symmetries(symmetries, surface_names, surface_numbers)

def surface_fitting(surfaces):
    for i, n1 in enumerate(np.array(surface_names)):
        d1 = surface_data[i]['d']
        a1 = surfaces[i] * d1
        for j, n2 in enumerate(np.array(surface_names)):
            d2 = surface_data[j]['d']
            a2 = surfaces[j] * d2
            nd = np.dot(n1, n2) / (np.linalg.norm(n1) * np.linalg.norm(n2))
            if a2 * nd > a1:
                surfaces[j] = int(round(a1 / (nd * d2)))

    return surfaces

def surface_centering(surfaces, basis='100', debug=0):
    if basis == '100':
        #Centering within the basis {[1,0,0], [0,1,0], [0,0,1]}
        dx = (surfaces[0] - surfaces[1]) // 2
        dy = (surfaces[2] - surfaces[3]) // 2
        dz = (surfaces[4] - surfaces[5]) // 2

        if (dx + dy + dz) % 2 == 1:
            dx -= 1

        if debug:
            print('(%i, %i, %i)' % (dx, dy, dz))
    elif basis == '110':
        #Centering within the basis {[1,1,0], [1,0,1], [0,1,1]}
        dl1 = ((surfaces[6] - surfaces[7]) // 4) * 2
        dl2 = ((surfaces[8] - surfaces[9]) // 4) * 2
        dl3 = ((surfaces[10] - surfaces[11]) // 4) * 2

        #Correction for the none orthogonality of the basis
        t1 = (dl1 != 0 and dl2 != 0 and dl3 == 0)
        t2 = (dl1 != 0 and dl2 == 0 and dl3 != 0)
        t3 = (dl1 != 0 and dl2 == 0 and dl3 != 0)

        if t1 or t2 or t3:
            d1 = (3 * dl1) // 2 - dl2 // 2 - dl3 // 2
            d2 = (3 * dl2) // 2 - dl1 // 2 - dl3 // 2
            d3 = (3 * dl3) // 2 - dl1 // 2 - dl2 // 2
        else:
            d1, d2, d3 = 0, 0, 0

        #Converting to '100' basis
        dx = (d1 + d2) // 2
        dy = (d1 + d3) // 2
        dz = (d2 + d3) // 2

        if debug:
            print ('(%i, %i, %i) -> (%i, %i, %i) -> (%i, %i, %i)' %
                   (dl1, dl2, dl3, d1, d2, d3, dx, dy, dz))
    else:
        dx, dy, dz = 0

    s = np.array(surfaces, int)
    ds = np.array([- dx, dx,
                   - dy, dy,
                   - dz, dz,
                   - dx - dy, dx + dy,
                   - dx - dz, dx + dz,
                   - dy - dz, dy + dz,
                   - dx + dy, dx - dy,
                   - dx + dz, dx - dz,
                   - dy + dz, dy - dz,
                   (-dx - dy - dz) // 2, (dx + dy + dz) // 2,
                   (dx - dy - dz) // 2, (-dx + dy + dz) // 2,
                   (-dx + dy - dz) // 2, (dx - dy + dz) // 2,
                   (-dx - dy + dz) // 2, (dx + dy - dz) // 2], int)

    if (s + ds >= 0).all():
        surfaces = s + ds

    return surfaces

#Definition of the neighbor environment
neighbor_names = [(0.5, 0.5, 0), (-0.5, -0.5, 0),
                  (0.5, 0, 0.5), (-0.5, 0, -0.5),
                  (0, 0.5, 0.5), (0, -0.5, -0.5),
                  (0.5, -0.5, 0), (-0.5, 0.5, 0),
                  (0.5, 0, -0.5), (-0.5, 0, 0.5),
                  (0, 0.5, -0.5), (0, -0.5, 0.5)]

neighbor_numbers = {}
for i, n in enumerate(neighbor_names):
    neighbor_numbers[n] = i

neighbor_positions = np.array(neighbor_names, dtype=float)

neighbor_cutoff = 0.8
neighbor_count = 12

neighbor_mapping = {0: 1, 1: 0, 2: 3, 3: 2, 4: 5, 5: 4, 
                    6: 7, 7: 6, 8: 9, 9: 8, 10: 11, 11: 10}

neighbor_symmetries = get_neighbor_symmetries(symmetries,
                                              neighbor_positions,
                                              neighbor_numbers)

#Definition of the atom types that is used based on the neighborlist
basetype_names = [(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0),
                  (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
                  (0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1),
                  (0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1),
                  (0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1), 
                  (0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1),
                  (0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1),
                  (0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1),
                  (0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1),
                  (0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0),
                  (1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0),
                  (0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0),
                  (1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0),
                  (0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1),
                  (1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1),
                  (0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1),
                  ]

basetype_data = [{'type': 0, 
                  'coordination': 0, 
                  'name': 'Free atom, Unknown'}, 
                 {'type': 1, 
                  'coordination': 12, 
                  'name': 'bulk'}, 
                 {'type': 2, 
                  'coordination': 8, 
                  'name': '100 surface'}, 
                 {'type': 3, 
                  'coordination': 7, 
                  'name': '110 surface (top), 111-111 edge, 110-111 edge'}, 
                 {'type': 4, 
                  'coordination': 11, 
                  'name': '110 surface (bottom)'}, 
                 {'type': 5, 
                  'coordination': 9, 
                  'name': '111 surface'}, 
                 {'type': 6, 
                  'coordination': 6, 
                  'name': '100-110 edge'}, 
                 {'type': 7, 
                  'coordination': 7, 
                  'name': '100-111 edge'}, 
                 {'type': 8, 
                  'coordination': 6, 
                  'name': '111-111-100 corner'}, 
                 {'type': 9, 
                  'coordination': 4, 
                  'name': '100 surface ad-atom'}, 
                 {'type': 10, 
                  'coordination': 5, 
                  'name': '110 surface ad-atom (bottom), A5 site'}, 
                 {'type': 11, 
                  'coordination': 3, 
                  'name': '111 surface ad-atom'}, 
                 {'type': 12, 
                  'coordination': 9, 
                  'name': '100 surface with ad-atom'}, 
                 {'type': 13, 
                  'coordination': 8, 
                  'name': '110 surface with ad-atom'}, 
                 {'type': 14, 
                  'coordination': 10, 
                  'name': '111 surface with ad-atom'}, 
                 {'type': 15, 
                  'coordination': 5, 
                  'name': 'B5 site'}, 
                 ]

type_count = len(basetype_names)

type_names = []
type_data = []
for i, n in enumerate(basetype_names):
    type_names.append(n)
    type_data.append(basetype_data[i])
    
    for sym in neighbor_symmetries:
        new_type = apply_neighbor_symmetry(n, sym)

        if not new_type in type_names:
            type_names.append(new_type)
            type_data.append(basetype_data[i])

type_numbers = {}
for i, n in enumerate(type_names):
    type_numbers[n] = i

data = {'symmetries': symmetries,
        'surface_names': surface_names,
        'surface_numbers': surface_numbers,
        'surface_data': surface_data,
        'surface_count': surface_count,
        'surface_mapping': surface_mapping,
        'surface_symmetries': surface_symmetries,
        'neighbor_positions': neighbor_positions,
        'neighbor_numbers': neighbor_numbers,
        'neighbor_count': neighbor_count,
        'neighbor_cutoff': neighbor_cutoff,
        'neighbor_mapping': neighbor_mapping,
        'neighbor_symmetries': neighbor_symmetries,
        'type_names': type_names,
        'type_numbers': type_numbers,
        'type_data': type_data,
        'type_count': type_count,
       }



"""
A simple test script.

Runs and times all scripts named ``*.py``.  The tests that execute
the fastest will be run first.
"""
from __future__ import print_function

import glob
import time
import pickle
import sys
import gc
try:
    import StringIO
except ImportError:
    # Python 3?
    import io as StringIO
from optparse import OptionParser
import asap3
import os
from ase.parallel import paropen

from asap3.mpi import world
master = world.rank == 0


if master:
    print("")
    print("*** Running ASAP Test Suite ***")
    print("")
    asap3.print_version(1)
    
brokentests = [
               'PrintMemory.py',  # Not really broken, but prints to stderr
               'EMTleak.py',      # Prints to stderr
               'Exception.py',    # Does not work with testsuite.
               # REALLY BROKEN TESTS
               'BrennerNanotube.py',  # Leaks memory (bug in numpy)
               'BrennerStress.py',    # Must be completed.
               'OpenKIM_AllModels.py',  # Tests the models, not Asap.  Some fail.
               'RDF2.py',
               'Hydrocarbons.py',
               'FieldPlotter.py',    # Fails on machines without gs or netpbm
               'bundleTrajPreserveZ.py',  # Fails on buildbot due to old ASE.
               'ConstraintInTraj.py',
               ]

slowtests = ['HooverNPT.py', 'HooverNPTtraj.py', 'Langevin.py',
             'LongLangevin.py', 'TinyLangevin.py',
             'parallelLennardJones.py',
             'parallelLangevin.py', 'parallelLongVerlet.py']

stdout = sys.stdout
stderr = sys.stderr
if not master:
    stdout = sys.stdout = open(("TestAll-proc%d.out" % (world.rank)), "w")

parser = OptionParser(usage='%prog [options] [tests]',
                      version='%prog 0.1')

parser.add_option('-x', '--exclude',
                  type='string', default=None,
                  help='Exclude tests (comma separated list of tests).',
                  metavar='test1.py,test2.py,...')

parser.add_option('-f', '--run-failed-tests-only',
                  action='store_true',
                  help='Run failed tests only.')

parser.add_option('-p', '--parallel',
                  action='store_true',
                  help='Run parallel tests only.')

parser.add_option('-s', '--slow',
                  action='store_true',
                  help='Also run slow tests.')

parser.add_option('-t', '--threads',
                  action='store_true',
                  help='Run with threading enabled.')

parser.add_option('-T', '--forcethreads',
                  action='store_true',
                  help='Run with threading enabled (4 threads forced).')

parser.add_option('-v', '--view-output',
                  action='store_true',
                  help="View output of all tests instead of redirecting them.")

parser.add_option('-L', '--leaktest',
                  action='store_true',
                  help="Enable leak testing.")

options, tests = parser.parse_args()

# Clear the arguments, so we do not trouble the tests.
del sys.argv[1:]

if options.parallel:
    path = 'parallel/'
    print("Running in parallel on %d CPUs." % (world.size,))
else:
    path = ''

if options.threads:
    asap3.AsapThreads()
    
if options.forcethreads:
    asap3.AsapThreads(4, force=True)
    
if len(tests) == 0:
    tests = glob.glob(path + '*.py')

    exclude = ['__init__.py', 'TestAll.py'] + brokentests
    if options.exclude is not None:
        exclude += options.exclude.split(',')

    if not options.slow:
        exclude += slowtests
        if master:
            print("")
            print("Skipping slow tests (use --slow to include them):")
            for x in slowtests:
                print("   ", x)
            print("")

    for test in exclude:
        if path + test in tests:
            tests.remove(path + test)
    
gc.set_debug(gc.DEBUG_SAVEALL)

# Read old timings if they are present:
try:
    timings = pickle.loads(open('timings.pickle', "rb").read())
except IOError:
    timings = {}

# Make a list of tests to do, and sort it with the fastest/new
# tests first:
tests = [(timings.get(test, 0.0), test) for test in tests]
tests.sort()

if options.run_failed_tests_only:
    tests = [(t, test) for t, test in tests if t == 0.0]

L = max([len(test) for told, test in tests])
print('-----------------------------------------------------------------')
print(' test', ' ' * (L - 4), 'result      time (old)')
print('-----------------------------------------------------------------')

leaktest = options.leaktest
garbage = []
failed = []
# Do tests:
for told, test in tests:
    
    if not options.view_output:
        sys.stdout = StringIO.StringIO()
        sys.stderr = StringIO.StringIO()

    print('%-*s' % (L + 2, test), end=' ', file=stdout)
    stdout.flush()

    ok = False
    
    module = test[:-3]
    if options.parallel:
        module = module.replace('/', '.')
        
    t = time.time()
    try:
        xxxx = __name__
        __name__ = "__main__"
        module = __import__(module, globals(), locals(), [])
        del module
        __name_ = xxxx
    except KeyboardInterrupt:
        failed.append(test)
        print('STOPPED!', file=stdout)
        print(('Hit [enter] to continue with next test, ' +
                                  '[ctrl-C] to stop.'), file=stdout)
        try:
            raw_input()
        except KeyboardInterrupt:
            break
        continue
    except AssertionError as msg:
        print('FAILED!', file=stdout)
        print(msg, file=stdout)
    except:
        print('CRASHED!', file=stdout)
        type, value = sys.exc_info()[:2]
        print(str(type) + ":", value, file=stdout)
    else:
        ok = True
        
    t = time.time() - t

    gc.collect()
    n = len(gc.garbage)
    if n > 0 and leaktest:
        print(' LEAK!', file=stdout)
        print(('Uncollectable garbage (%d object%s)' %
                                  (n, 's'[:n > 1])), file=stdout)
        print("Further leak testing not possible", file=stdout)
        print(file=stdout)
        print("The first 10 leaked objects are:", file=stdout)
        for o in gc.garbage[:10]:
            print(repr(o), type(o), type(o).__module__, file=stdout)
            del o
        leaktest = False
        ok = False
    if ok:
        print('  OK     %7.3f (%.3f)' % (t, told), file=stdout)
    else:
        failed.append(test)
        if not options.view_output:
            out = sys.stdout.getvalue()
            if len(out) > 0 and master:
                open(test + '.output', 'w').write(out)
            err = sys.stderr.getvalue()
            if len(err) > 0 and master:
                open(test + '.error', 'w').write(err)
        t = 0
    timings[test] = t
        
if not options.view_output:
    sys.stdout = stdout
    sys.stderr = stderr

print('-----------------------------------------------------------------')

if len(tests) > 1:
    print()
    if len(failed) == 0:
        print('All tests passed!')
    elif len(failed) == 1:
        print('One test out of %d failed: %s' % (len(tests), failed[0]))
    else:
        print('%d tests out of %d failed:'% (len(failed), len(tests)))
        for test in failed:
            print(' ', test)
    print()

# Save new timings:
open('timings.pickle', 'wb').write(pickle.dumps(timings, protocol=2))

# Report the error to build bot.
failedname = "TestAll-failed.txt"
if len(failed):
    failedfile = paropen(failedname, "w")
    failedfile.write(str(len(failed))+'\n')
    failedfile.close()
    sys.exit(1)
else:
    if os.path.exists(failedname) and master:
        os.remove(failedname)



from asap3.MonteCarlo.Moves.Base import SingleMove, ShakeMove, \
                                        BallMove, ShellMove, \
                                        BrownianMove, ExchangeMove, \
                                        CompressMove
from asap3.MonteCarlo.Moves.Surface import SurfaceMove




"""Test local crystal structure analysis (CNA and Coordination number)."""
from __future__ import print_function

from asap3 import *
from asap3.analysis.localstructure import RestrictedCNA, CoordinationNumbers
from asap3.testtools import ReportTest
from asap3.mpi import world
from ase.build import bulk
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallel%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

nbl_cutoff = 4.5

def makesystem():
    atoms = bulk('Cu')
    atoms = atoms.repeat((15, 15, 15))
    if cpulayout:
        atoms = atoms.repeat(cpulayout)
    atoms.set_pbc((True, False, True))
    #atoms.set_pbc((False, True, True))
    del atoms[87]   #Create point defect
    return atoms

for usepot in (True, False):
    # Make serial nb list.
    seratoms = makesystem()
    sernblist = FullNeighborList(nbl_cutoff, seratoms)

    # Make parallel nb list
    if ismaster:
        atoms = makesystem()
    else:
        atoms = None
    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpulayout)
    natoms = atoms.get_number_of_atoms()
    if usepot:
        atoms.set_calculator(EMT())
        old_energy = atoms.get_potential_energy()
        print(atoms.get_calculator().get_cutoff())
        print(len(atoms), atoms.get_number_of_atoms())
        #assert atoms.get_calculator().get_cutoff() >= nbl_cutoff
    
    print("Testing parallel neighbor locator (usepot = {0})".format(usepot))
    nblist = FullNeighborList(nbl_cutoff, atoms)
    print("Length of neighbor locator", len(nblist), len(atoms))
    print(nblist[-1])
    if isparallel:
        ids = atoms.get_ids()
    else:
        ids = range(len(atoms))
    maxlen = -1
    minlen = 1e100
    for i in range(len(atoms)):
        l = len(nblist[i])
        l_exp = len(sernblist[ids[i]])
        ReportTest("NBlist length (i={0}, id={1})".format(i, ids[i]), 
                   l, l_exp, 0, silent=True)
        if l > maxlen:
            maxlen = l
        if l < minlen:
            minlen = l
        if l == l_exp:
            snbl = set((n for n in sernblist[ids[i]]))
            for nb in nblist[i]:
                if nb < len(atoms):
                    assert ids[nb] in snbl

    print("Neighbor list length:", minlen, '-', maxlen)
    if usepot:
        atoms[1].position += np.array((1e-5,0,0))
        ReportTest("Energy", atoms.get_potential_energy(), old_energy, 1e-4)
 

#ReportTest.Summary()



from ase.optimize.fire import FIRE as _FIRE
from asap3.optimize.optimize import asap_Optimizer_mixin
from asap3.mpi import world
import numpy as np

class FIRE(asap_Optimizer_mixin, _FIRE):
    vtag = "fire_v"

    def initialize(self):
        if self.atoms.has(self.vtag):
            self.atoms.set_array(self.vtag, None)

    def read(self):
        if world.size > 1:
            raise NotImplementedError("Restarting is not implemented for FIRE in parallel mode (ASAP).")
        v, self.dt = self.load()
        self.atoms.set_array(self.vtag, v)
       
    def step(self,f=None):
        atoms = self.atoms
        if f is None:
            f = atoms.get_forces()
        if not atoms.has(self.vtag):
            v = np.zeros((len(atoms), 3))
        else:
            v = atoms.get_array(self.vtag)
            vf = np.vdot(f, v)
            if world.size > 1:
                vf = world.sum(vf)
            if vf > 0.0:
                vv = np.vdot(v, v)
                ff = np.vdot(f, f)
                if world.size > 1:
                    vv = world.sum(vv)
                    ff = world.sum(ff)
                v = (1.0 - self.a) * v + self.a * f / np.sqrt(
                    ff) * np.sqrt(vv)
                if self.Nsteps > self.Nmin:
                    self.dt = min(self.dt * self.finc, self.dtmax)
                    self.a *= self.fa
                self.Nsteps += 1
            else:
                v[:] *= 0.0
                self.a = self.astart
                self.dt *= self.fdec
                self.Nsteps = 0

        v += self.dt * f
        dr = self.dt * v
        drdr = np.vdot(dr, dr)
        if world.size > 1:
            drdr = world.sum(drdr)
        normdr = np.sqrt(drdr)
        if normdr > self.maxmove:
            dr = self.maxmove * dr / normdr
        r = atoms.get_positions()
        atoms.set_positions(r + dr)
        atoms.set_array(self.vtag, v)
        self.dump((v, self.dt))
        
    def dump(self, data):
        # Can only restart in serial mode
        if world.size == 1:
            _FIRE.dump(self, data)



#!/usr/bin/env python
"""
Tests that polyhedral template matching works on the simplest possible system.

Name: ptm.py

Description: Part of the Asap test suite.  Tests the analysis.ptm module

Usage: python ptm.py

Expected result: The output should end with 'ALL TESTS SUCCEEDED'.
"""
from __future__ import print_function

from ase.lattice.cubic import FaceCenteredCubic, BodyCenteredCubic
from ase.lattice.hexagonal import HexagonalClosedPacked
from ase.build import bulk
from asap3.testtools import ReportTest
from asap3.analysis import PTM
from asap3 import MakeParallelAtoms
from asap3.mpi import world
import numpy as np
#from ase.visualize import view

if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

def distribute(atoms):
    if world.rank != 0:
        atoms = None
    return MakeParallelAtoms(atoms, cpulayout)

atoms = distribute(FaceCenteredCubic("Cu", size=(14,14,14), pbc=False))
#x = atoms.get_positions()
#atoms.set_positions(x + 0.01 * np.random.standard_normal(x.shape))

data = PTM(atoms, rmsd_max=0.1, calculate_strains=True)
print("DONE")
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 13*13*13*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 13*13*13*4, 0)

data = PTM(atoms, rmsd_max=0.1, calculate_strains=True, quick=True)
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 13*13*13*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 13*13*13*4, 0)

data = PTM(atoms, rmsd_max=0.1, target_structures=('fcc', 'hcp'))
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 13*13*13*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 13*13*13*4, 0)

atoms = distribute(BodyCenteredCubic("Fe", size=(14,14,14), pbc=(True,True,False)))
data = PTM(atoms, rmsd_max=0.001)
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of BCC atoms", cnt[3], 14*14*12*2, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 14*14*12*2, 0)

atoms = distribute(HexagonalClosedPacked("Mg", size=(14,14,14), pbc=(True,True,False)))
cutoff = atoms.get_cell()[0,0] / 4 / np.sqrt(3) * 0.95
print("Cutoff", cutoff)
data = PTM(atoms, rmsd_max=0.01, cutoff=cutoff)  # Need a slightly larger rmsd as c/a deviates from ideal.
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 14*14*13*2, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 14*14*13*2, 0)
#atoms.set_tags(data['structure'])
#view(atoms)

atoms = bulk("Mg", orthorhombic=True)
atoms = atoms.repeat((14,14,14))
atoms.set_pbc((True, True, False))
atoms = distribute(atoms)
data = PTM(atoms, rmsd_max=0.01)  # Need a slightly larger rmsd as c/a deviates from ideal.
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 14*14*13*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], atoms.get_number_of_atoms() - 14*14*13*4, 0)

# Make a combined system

atoms = bulk("Mg", orthorhombic=True)
atoms = atoms.repeat((14,8,14))

atoms2 = bulk("Fe", orthorhombic=True).repeat((6,6,6))
uc1 = atoms.get_cell()
uc2 = atoms2.get_cell()
factor1 = uc1[0,0] / uc2[0,0]
factor2 = uc1[1,1] / uc2[1,1]
uc2 = uc2 * np.array((factor1, factor2, 0.5*(factor1+factor2)))
atoms2.set_cell(uc2, scale_atoms=True)
# Combine unit cell, stack BCC atoms on top of HCP atoms.
uc3 = np.array(uc1)
dz = uc3[2,2]
uc3[2,2] += uc2[2,2]
atoms.set_cell(uc3, scale_atoms=False)
atoms2.set_positions(atoms2.get_positions() + np.array((0.1,0.1,dz))[np.newaxis,:])
atoms.extend(atoms2)
atoms = distribute(atoms)

data = PTM(atoms, rmsd_max=0.005)
cnt = np.bincount(data['structure'])
world.sum(cnt)
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 14*8*13*4, 0)
ReportTest("Number of BCC atoms", cnt[3], 6*6*4*2, 0)
ReportTest("Number of FCC atoms", cnt[1], 0, 0)
ReportTest("All classes of atoms", cnt[:4].sum(), atoms.get_number_of_atoms(), 0)

#atoms.set_tags(data['structure'])
#view(atoms)

ReportTest.Summary()



from __future__ import print_function
from numpy import *
#from Scientific.Statistics import *
from asap3 import *
from asap3.md.npt import NPT
from asap3.md.langevin import Langevin
from asap3.io.trajectory import Trajectory
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.mpi import world
from ase import units
import sys

T_goal = 300 # K
p_goal = 1 # GPa
bulk = 137 # Gpa
cp = 24.43 # J / (K * mol)
step1 = 25

if len(sys.argv) == 1 or sys.argv[0].endswith("TestAll.py"):
    out1 = "testNPT-1.out"
    out2 = "testNPT-2.out"
    ttime = 25
    ptime = 75
elif len(sys.argv) == 3:
    ttime = float(sys.argv[1])
    ptime = float(sys.argv[2])
    out1 = "testNPT-%d-%d-1.out" % (ttime, ptime)
    out2 = "testNPT-%d-%d-2.out" % (ttime, ptime)
else:
    raise RuntimeError("Expected zero or two parameter to the script!" + str(sys.argv))

ismaster = world.rank == 0
if world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 4:
    cpulayout = [2,2,1]
elif world.size == 8:
    cpulayout = [2,2,2]
else:
    raise RuntimeError("Cannot run on %i CPUs" % (world.size,))

if ismaster:
    print_version(1)

if not hasattr(NPT, '_npt_version'):
    print("Skipping test: NP dynamics does not work in parallel with this old version of ASE.")
else:
    if ismaster:
        atoms = FaceCenteredCubic(size=(30,15,15), symbol="Cu", pbc=True)
    else:
        atoms = None
    atoms = MakeParallelAtoms(atoms, cpulayout)
    atoms.set_calculator(EMT())
    print("Number of atoms:", atoms.get_number_of_atoms())

    print("Heating to %d K using Langevin" % T_goal)
    lgv = Langevin(atoms, 5 * units.fs, temperature=2*T_goal*units.kB, friction=0.05)

    while atoms.get_kinetic_energy() < 1.5 * atoms.get_number_of_atoms() * T_goal * units.kB:
        lgv.run(5)
        T = atoms.get_kinetic_energy() / (1.5 * atoms.get_number_of_atoms() * units.kB)
        print("Temperature is now %.2f K" % (T,))
    print("Desired temperature reached!")

    lgv.set_temperature(T_goal*units.kB)

    for i in range(2):
        lgv.run(20)
        s = atoms.get_stress()
        p = -(s[0] + s[1] + s[2])/3.0 / units.GPa
        T = atoms.get_kinetic_energy() / (1.5 * atoms.get_number_of_atoms() * units.kB)
        print("Pressure is %f GPa, desired pressure is %f GPa (T = %.2f K)" % (p, p_goal, T))
        dv = (p - p_goal) / bulk
        print("Adjusting volume by", dv)
        cell = atoms.get_cell()
        atoms.set_cell(cell * (1.0 + dv/3.0))

    T = atoms.get_kinetic_energy() / (1.5 * atoms.get_number_of_atoms() * units.kB)
    print("Temperature is now %.2f K" % (T,))

    stressstate = array([-2, -1, 0, 0, 0, 0])*p_goal*units.GPa
    dyn = NPT(atoms, 5 * units.fs, T_goal*units.kB, stressstate,
              ttime*units.fs, (ptime*units.fs)**2 * bulk * units.GPa)
    traj = Trajectory("NPT-atoms.traj", "w", atoms)
    #dyntraj = ParallelHooverNPTTrajectory("NPT-dyn-traj.nc", dyn, interval = 50)
    dyn.attach(traj, interval=50)
    #dyn.Attach(dyntraj)

    out = open(out1, "w")

    temp = []
    pres = []
    vol = []
    for i in range(step1):
        dyn.run(5)
        T = atoms.get_kinetic_energy() / (1.5 * atoms.get_number_of_atoms() * units.kB)
        s = atoms.get_stress() / units.GPa
        p = -(s[0] + s[1] + s[2])/3.0
        out.write("%5.2f %5.2f %7.5f      %7.5f %7.5f %7.5f %7.5f %7.5f %7.5f\n" %
                  ((dyn.get_time(), T, p)+tuple(s)))
        out.flush()
        cell = atoms.get_cell()
        v = cell[0,0] * cell[1,1] * cell[2,2]
        temp.append(T)
        pres.append(p)
        vol.append(v)
        if i % 5 == 0:
            print(i,"/",step1)

    del atoms, dyn, traj
    #del dyntraj




from __future__ import print_function
#import production

### Job name
#PBS -N LongVerlet

### Mail to user
#PBS -m ae

### Queue name (small, medium, long, verylong)
#PBS -q verylong

### Number of nodes
#PBS -l nodes=1:xeon:ppn=8


from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from pickle import *
from numpy import *
import sys, os, time
from asap3.testtools import *
from asap3.mpi import world

debug = 0
if debug == 1:
    DebugOutput("longverlet%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

writetraj = True
#set_verbose(1)


ismaster = world.rank == 0
isparallel = world.size != 1
if ismaster:
    print_version(1)
    
if world.size == 1:
    cpulayouts = [None]
elif world.size == 2:
    cpulayouts = [[2,1,1], [1,2,1]]
elif world.size == 3:
    cpulayouts = [[3,1,1], [1,1,3]]
elif world.size == 4:
    cpulayouts = [[4,1,1], [2,1,2]]
elif world.size == 8:
    cpulayouts = [[8,1,1], [4,2,1], [2,2,2]]
elif world.size == 16:
    cpulayouts = [[8,1,2], [4,2,2]]
else:
    raise RuntimeError("No cpu layout for %d CPUs" % (world.size,))

e_start_dict = {}
for cpulayout in cpulayouts:
    for boundary in ((1,1,1), (0,0,0), (0,1,1), (1,0,0)):
        print ("CPU Layout: %s.  Periodic boundary conditions: %s."
               % (str(cpulayout), str(boundary)))
        if ismaster:
            atoms = FaceCenteredCubic(size=(160,20,20), symbol="Cu",
                                      pbc=boundary, latticeconstant=3.61*1.04)
        else:
            atoms = None
        if isparallel:
            atoms = MakeParallelAtoms(atoms, cpulayout)
        natoms = atoms.get_number_of_atoms()
        atoms.set_calculator(EMT())
        MaxwellBoltzmannDistribution(atoms, 3000*units.kB)
        if ismaster:
            print("Initializing")
        e_st_pot = atoms.get_potential_energy()/natoms
        try:
            e_start_pot = e_start_dict[boundary]
        except KeyError:
            e_start_pot = e_start_dict[boundary] = e_st_pot
        else:
             ReportTest("Initial energy ", e_st_pot, e_start_pot, 1e-4,
                        silent=True)
        dyn = VelocityVerlet(atoms, logfile="-", dt=3*units.fs, loginterval=1)
        dyn.run(50)
        e_start = (atoms.get_potential_energy() 
                   + atoms.get_kinetic_energy())/natoms
        if ismaster:
            print("Running")
        dyn = VelocityVerlet(atoms, dt=5*units.fs)
        logger = MDLogger(dyn, atoms, '-', peratom=True)
        logger()
        dyn.attach(logger, interval=25)
        if writetraj:
            if cpulayout is None:
                tname = "longVerlet-serial-%d-%d-%d.traj" % tuple(boundary)
            else:
                tname = ("longVerlet-%d-%d-%d--%d-%d-%d.traj" %
                         (tuple(cpulayout) + tuple(boundary)))
            traj = Trajectory(tname, "w", atoms)
            traj.write()
            dyn.attach(traj, interval=1000)
        
        for i in range(100):
            dyn.run(100)
            if i % 5 == 4:
                print(i+1, "%")
            e = (atoms.get_potential_energy() + atoms.get_kinetic_energy())/natoms
            ReportTest("Step "+str(i), e, e_start, 1e-4, silent=True)

ReportTest.Summary()




# -*- coding: utf-8 -*-

from __future__ import print_function
from asap3 import FullNeighborList
import numpy as np
import collections

def findclusters(atoms, tag, cutoff, arrayname='cluster', tags=None,
                 returnclusters=False):
    """Find clusters of atoms with a given tag.

    Find connected clusters of atoms with a given tag.  Each cluster
    is assigned a cluster number starting with 0, the cluster numbers
    are assigned to an array named 'cluster' on the atoms.  The name
    of the array can be changed.

    In addition to setting the 'cluster' array on the atoms, the
    function returns a list of cluster sizes.

    If the returnclusters parameter is set to True, the function
    returns two objects: a list of cluster sizes, and a map mapping
    cluster sizes to lists of clusters; each such cluster is itself
    a list of atom indices.
    
    Parameters:

    atoms: The atoms to be analyzed.

    tag: The tag identifying interesting atoms.  atoms.get_tags() is
        used to read the tags from the atoms, and only atoms matching
        'tag' are considered in the clustering.  Set tag=None to
        consider all atoms (for a dense sample this gives a single
        cluster spanning the entire system).

    cutoff: The cutoff distance used to identify if two atoms are
        neighbors.

    arrayname='cluster': The name of the array on the atoms where the
        cluster ID is stored.  Changed to 'dislocation' by the
        dislocation module.

    tags=None: An array to be used as tags instead of atoms.get_tags().
        Ignored if tag=None.

    returnclusters=False: If True, the function also returns a dictionary
        mapping cluster sizes to lists of clusters.
    """
    if tag is None:
        # All atoms match
        tag = 1
        tags = np.ones(len(atoms), np.int8)
    if tags is None:
        tags = atoms.get_tags()
    # Find the interesting atoms
    interesting = np.nonzero(tags == tag)[0]
    # Assign all interesting atoms to cluster -1 (unassigned), and all
    # other atoms to cluster -2 (not considered)
    cluster = -2 * np.ones(len(atoms))
    cluster[interesting] = -1
    print(len(atoms))
    print(interesting.shape)
    print(cluster.shape)
    if returnclusters:
        clustermap = collections.defaultdict(list)
        
    # Loop over interesting atoms not assigned so far.  Use flood fill
    # to find their cluster
    neighborlist = FullNeighborList(cutoff, atoms, driftfactor=0.0)
    print("Neighbor list ready.")
    next_cluster = 0
    cluster_sizes = []
    # Loop over the indices of the interesting atoms
    for i in interesting:
        if cluster[i] == -1:
            # Not yet identified
            in_cluster = flood_fill(neighborlist, i, next_cluster, cluster)
            n = len(in_cluster)
            cluster_sizes.append(n)
            if returnclusters:
                clustermap[n].append(in_cluster)
            next_cluster += 1
            if (next_cluster % 10000 == 0):
                print("  cluster {0}:  size = {1},  longest = {2},  remaining = {3}".format(
                    next_cluster, n, max(cluster_sizes), (cluster == -1).sum()))
    if returnclusters:
        return cluster_sizes, clustermap
    else:
        return cluster_sizes

def flood_fill(nblist, idx, cluster_id, cluster_array):
    """Flood fill algorithm used to identify atoms in this cluster.

    Parameters:

    nblist: An up-to-date neighborlist.

    idx: The index of the starting atom in the cluster.

    cluster_id: The ID being assigned to atoms in this cluster.

    cluster_array: The IDs of all atoms.  Unassigned atoms have ID -1.
        This array is modified by the algorithm.

    Returns:

    A list of the atoms in the cluster.  In addition, cluster_array is
    modified.
    """
    assert(cluster_array[idx] == -1)
    cluster_array[idx] = cluster_id
    queue = collections.deque()
    queue.append(idx)
    n = 1   # We already have the first atom
    result = [idx]
    while queue:
        i = queue.popleft()
        for nb in nblist[i]:
            if cluster_array[nb] == -1:
                n += 1
                cluster_array[nb] = cluster_id
                result.append(nb)
                queue.append(nb)
    return result

    
if __name__ == '__main__':
    import sys
    import ase.io
    import matplotlib.pyplot as plt
    
    filename = sys.argv[1]
    if len(sys.argv) >= 3:
        frame = int(sys.argv[2])
    else:
        frame = -1
    print("Reading frame {0} of {1}".format(frame, filename))
    atoms = ase.io.read(filename, frame)
    cutoff = 3.615 * (1/np.sqrt(2) + 1) / 2
    print("Cutoff is {0} A".format(cutoff))
    data = findclusters(atoms, 8, cutoff)
    w_histo = np.bincount(data)
    w_histo *= np.arange(len(w_histo))
    w_histo = w_histo[:25]
    plt.bar(np.arange(len(w_histo)), w_histo)
    #plt.axis([0, 800, 0, 5000])
    plt.show()

    



"""Module for the creation of nanocrystalline samples."""


from .nanocrystalline import make_nanocrystal
from .dislocated import make_dislocated_nanocrystal
from .grain_structures import bcc_grains, random_rotations, \
     lloyd_step, evaluate_voronoi_cells, plot_voronoi_cells
from .quat_utils import random_rotation_matrix
from .energy import minimize_energy



import numpy as np
import numbers

from ase.units import kB, fs
from ase.md import Langevin

from asap3 import CoordinationNumbers

#from asap3.MonteCarlo.Moves.Surface import SurfaceMove

class Move:
    def __init__(self):
        self.optimizer = None
        self.atoms = None

    def __call__(self, atoms=None):
        self.set_atoms(atoms)
        #self.backup()

    def get_name(self):
        return self.__class__.__name__

    def set_optimizer(self, optimizer=None):
        if optimizer is not None:
            self.optimizer = optimizer

    def set_atoms(self, atoms):
        if atoms is not None:
           self.atoms = atoms

    def backup(self, idx=None):
        """Save positions of atoms to be moved.
        
        idx should be None to save all atoms, or either an integer or an 
        array/list of intergers specifying which atoms to save. 
        """
        if idx is None:
            self.backup_positions = self.atoms.get_positions()
        else:
            self.backup_positions = self.atoms.get_positions()[idx]
        self.backup_idx = idx

    def accept(self):
        pass

    def reject(self):
        if self.backup_idx is None:
            self.atoms.set_positions(self.backup_positions)
        elif isinstance(self.backup_idx, numbers.Integral):
            self.atoms[self.backup_idx].position = self.backup_positions
        else:
            # Set atoms one by one to trigger possible MC optimizations
            # in potential.
            for i, r in zip(self.backup_idx, self.backup_positions):
                self.atoms[i].position = r

    def log(self):
        return ""

class SingleMove(Move):
    """Displaces one atom randomly within a sphere around it self."""
    def __init__(self, rmax):
        self.rmax = rmax
        Move.__init__(self)

    def __call__(self, atoms):
        self.set_atoms(atoms)

        natoms = len(self.atoms)
        n = RandChoice(np.arange(natoms))
        d = RandSphere(1, 0.0, self.rmax)

        self.backup(n)
        self.atoms[n].position += d[0]

class ShakeMove(Move):
    """Displaces all atoms randomly within a sphere around them self."""
    def __init__(self, rmax):
        self.rmax = rmax
        Move.__init__(self)

    def __call__(self, atoms):
        self.set_atoms(atoms)
        self.backup()
        natoms = len(atoms)
        positions = atoms.get_positions()
        d = RandSphere(natoms, 0.0, self.rmax)
        positions += d
        atoms.set_positions(positions)

class BallMove(Move):
    """Moves one atom randomly within the cluster modelled by an
    ellipsoid."""
    def __call__(self, atoms):
        self.set_atoms(atoms)

        natoms = len(atoms)
        positions = atoms.get_positions()
        cn = CoordinationNumbers(atoms)

        # Determine surface atoms
        mask = (cn < 11) * (cn > 5)
        if mask.sum() < 12:
            return positions   # XXXX This cannot be right!

        # Fit surface to ellipsoid
        (A, C, Q) = FitEllipsoid(positions[mask])

        # Move atom
        n = RandChoice(np.arange(natoms))
        d = RandEllipsoid(1, (0, A[0]), (0, A[1]), (0, A[2]))
        self.backup(n)
        atoms[n].position = np.dot(Q, d + C)

class ShellMove(Move):
    """Moves n low coordinated atoms randomly around the surface of the
    cluster. The surface is defined as an ellipsiod."""
    def __init__(self, width, steps=1):
        self.width = width
        self.steps = steps
        Move.__init__(self)

    def __call__(self, atoms):
        self.set_atoms(atoms)
        self.backup()
        
        natoms = len(atoms)
        positions = atoms.get_positions()
        cn = CoordinationNumbers(atoms)

        # Determine surface atoms
        mask = (cn < 11) * (cn > 5)
        if mask.sum() < 12:
            return positions

        # Fit surface to ellipsoid
        (A, C, Q) = FitEllipsoid(positions[mask])
        Amin = A - self.width / 2
        Amax = A + self.width / 2
        a = (Amin[0], Amax[0])
        b = (Amin[1], Amax[1])
        c = (Amin[2], Amax[2])

        # Move atoms
        n = 1
        mask = (cn <= n)
        while mask.sum() < self.steps:
            n += 1
            mask = (cn <= n)

        moved = []
        while len(moved) < self.steps:
            n = RandChoice(np.arange(natoms)[mask])
            if n not in moved:
                moved.append(n)
                d = RandEllipsoid(1, a, b, c)
                positions[n] = np.dot(Q, d + C)

        atoms.set_positions(positions)

class CompressMove(Move):
    def __init__(self, cmax=0.3, cmin=0.2, steps=2, rotate=True):
        if cmin > cmax:
            raise ValueError("cmax must be greater than cmin.")
        self.cmax = cmax
        self.cmin = cmin
        self.steps = steps
        self.rotate = rotate
        Move.__init__(self)

    def __call__(self, atoms):
        self.set_atoms(atoms)
        self.backup() 

        natoms = len(atoms)
        positions = atoms.get_positions()
        center = atoms.get_center_of_atoms()

        for i in range(self.steps):
            if self.rotate:
                positions = RotatePositions(positions, RandSphere(),
                                            [0.0, 0.0, 1.0], center)
            c = np.random.uniform(1.0 - self.cmax, 1.0 - self.cmin, natoms)
            positions[:,2] = c * (positions[:,2] - center[2]) + center[2]

        atoms.set_positions(positions)

class BondMove(Move):
    def __call__(self):
        pass

class BrownianMove(Move):
    """Makes a small Langevin simulation."""
    def __init__(self, steps, timestep, temperature, friction):
        self.steps = steps
        self.timestep = timestep * fs
        self.temp = temperature * kB
        self.friction = friction
        Move.__init__(self)

    def __call__(self, atoms):
        self.set_atoms(atoms)
        self.backup()

        # Start med en temperatur!
        dyn = Langevin(atoms, self.timestep, self.temp, self.friction)
        dyn.run(self.steps)

class ExchangeMove(Move):
    """Exchange two atoms with different type."""
    def __call__(self, atoms):
        self.set_atoms(atoms)

        natoms = len(atoms)
        numbers = atoms.get_atomic_numbers()
        positions = atoms.get_positions()

        bin = np.bincount(numbers)
        num = RandChoice(np.arange(len(bin))[bin > 0])

        mask1 = (numbers == num)
        mask2 = np.logical_not(mask1)

        if not (mask2 == True).any():
            raise Warning("Cannot use exchange move with only one element.")

        n1 = RandChoice(np.arange(natoms)[mask1])
        n2 = RandChoice(np.arange(natoms)[mask2])

        p1 = positions[n1].copy()
        p2 = positions[n2].copy()
        self.backup([n1,n2])
        atoms[n1].position = p2
        atoms[n2].position = p1

### Helper functions ###
def RandSphere(length=1, rmin=0.0, rmax=1.0):
    return RandEllipsoid(length, (rmin, rmax), None, None)

def RandEllipsoid(length=1, a=(0.9, 1.1), b=(0.9, 1.1), c=(0.9, 1.1)):
    if isinstance(a, (list, tuple)):
        (amin, amax) = a
        a = np.random.uniform(amin, amax, length)
    else:
        a = a * np.ones(length)

    if b is None:
        b = a
    elif isinstance(b, (list, tuple)):
        (bmin, bmax) = b
        b = np.random.uniform(bmin, bmax, length)
    else:
        b = b * np.ones(length)

    if c is None:
        c = a
    elif isinstance(c, (list, tuple)):
        (cmin, cmax) = c
        c = np.random.uniform(cmin, cmax, length)
    else:
        c = c * np.ones(length)

    t = np.random.uniform(0.0, np.pi, length)
    p = np.random.uniform(0.0, 2*np.pi, length)

    x = a * np.sin(t) * np.cos(p)
    y = b * np.sin(t) * np.sin(p)
    z = c * np.cos(t)

    xyz = np.zeros((length, 3))
    xyz[:,0] = x
    xyz[:,1] = y
    xyz[:,2] = z

    if length > 1:
        return xyz
    else:
        return xyz[0]

def RotatePositions(positions, u, v, center=[0, 0, 0]):
        # Rotate the vector u into v
        center = np.array(center, float)
        pos = positions - center

        u = np.array(u, float)
        u /= np.linalg.norm(u)

        v = np.array(v, float)
        v /= np.linalg.norm(v)

        c = np.dot(u, v)
        w = np.cross(u, v)
        s = np.linalg.norm(w)
        # In case *u* and *v* are parallel, np.cross(u, v) vanish
        # and can't be used as a rotation axis. However, in this
        # case any rotation axis perpendicular to v2 will do.
        eps = 1e-7
        if s < eps:
            w = np.cross((0, 0, 1), v)
            s = np.linalg.norm(w)
        if s < eps:
            w = np.cross((1, 0, 0), v)
            s = np.linalg.norm(w)
        assert s >= eps
        if s > 0:
            w /= s
        return (c * pos - np.cross(pos, s * w) +
                np.outer(np.dot(pos, w), (1.0 - c) * w) + center)

def RandChoice(a):
    n = np.random.random_integers(0, len(a) - 1)
    return a[n]

def FitEllipsoid(xyz):
    """Fitting an ellipsoid based on Qingde Li et al., "Least Squares
    Ellipsoid Specific Fitting", 2004."""
    # Set up matrices
    C = np.array([[-1, 1, 1, 0, 0, 0],
                  [1, -1, 1, 0, 0, 0],
                  [1, 1, -1, 0, 0, 0],
                  [0, 0, 0, -4, 0, 0],
                  [0, 0, 0, 0, -4, 0],
                  [0, 0, 0, 0, 0, -4]])
    Ci = np.linalg.inv(C)

    X = np.array([xyz[:,0]**2, xyz[:,1]**2, xyz[:,2]**2,
                  2*xyz[:,1]*xyz[:,2], 2*xyz[:,0]*xyz[:,2],
                  2*xyz[:,0]*xyz[:,1],
                  2*xyz[:,0], 2*xyz[:,1], 2*xyz[:,2],
                  np.ones(len(xyz))])

    XXT = np.dot(X, X.T)
    S11 = XXT[:6,:6].copy()
    S12 = XXT[:6,6:].copy()
    S12T = XXT[6:,:6].copy()
    S22i = np.linalg.inv(XXT[6:,6:])

    # Solve fitting equation
    eig = np.linalg.eig(np.dot(Ci, S11 - np.dot(S12, np.dot(S22i, S12T))))
    v = eig[1][:, eig[0].argmax()]
    c = np.append(v, -np.dot(np.dot(S22i, S12T), v))

    # Reduce the quadratic equation
    A = np.array([[c[0], c[5], c[4]],
                  [c[5], c[1], c[3]],
                  [c[4], c[3], c[2]]])
    B = np.array([c[6], c[7], c[8]])
    J = c[9]

    eig = np.linalg.eig(A)
    A1 = eig[0]
    Q = eig[1]
    B1 = np.dot(B, Q)
    K = (B1**2 / A1).sum() - J

    # Return semi-axes, center and rotation matrix
    return (np.sqrt(K/A1), -B1/A1, Q)




"""Fix epydoc documentation.

The function fix is used to fix epydoc documentation by fooling
epydoc into documenting objects although they are really documented in
a submodule.
"""

import inspect, sys

if 'epydoc' in sys.modules:
    def fix(*objects):
        """Fix epydoc documentation for objects.
        
        The function fixdoc is used to fix epydoc documentation by
        fooling epydoc into documenting objects although they are
        really documented in a submodule.
        
        Usage:
    
        import Asap.fixepydoc
        from submodule import a, b
        Asap.fixepydoc.fix(a, b)
        
        This will cause epydoc to document objects (classes,
        functions, ...) a and b as if they were actually defined here.
        If epydoc is not running, nothing happens.
        
        Do not call fix on the same object in more than one module.
        """
        caller = inspect.getmodule(inspect.currentframe().f_back).__name__
        assert caller is not None
        for o in objects:
            assert hasattr(o, '__module__')
            o.__module__ = caller
else:
    def fix(*objects):
        "Fix epydoc documentation: Do nothing as epydoc is not running."
        pass
    
            



from __future__ import print_function
#import production

from asap3 import *
from asap3.md.verlet import VelocityVerlet
from pickle import *
from numpy import *
import sys, os, time
from asap3.testtools import *
from asap3.mpi import world

openkimmodel = "EMT_Asap_Standard_JacobsenStoltzeNorskov_1996_Cu__MO_396616545191_001"

if sys.version_info[0] >= 3:
    picklehack = {'encoding': 'latin1'}
else:
    picklehack = {}

if OpenKIMsupported:
    try:
        calc = OpenKIMcalculator(openkimmodel)
    except AsapError as oops:
        if oops.args[0].startswith('Failed to initialize OpenKIM model'):
            print("OpenKIM model {} not installed - skipping test.".format(openkimmodel))
            calc = None
        else:
            raise
    
if OpenKIMsupported and calc is not None:
    debug = 0
    if debug == 1:
        DebugOutput("parallelopenkim%d.log", nomaster=True)
    elif debug == 2:
        time.sleep(world.rank)
        print("PID:", os.getpid())
        time.sleep(20)

    print_version(1)
    #set_verbose(1)

    ismaster = world.rank == 0
    isparallel = world.size != 1
    if world.size == 1:
        cpulayout = None
    elif world.size == 2:
        cpulayout = [2,1,1]
    elif world.size == 3:
        cpulayout = [1,3,1]
    elif world.size == 4:
        cpulayout = [2,1,2]

    def report():
        for i in range(world.size):
            if i == world.rank:
                print("Data on processor", i)
                for key in atoms.arrays.keys():
                    print("  ", key, atoms.arrays[key].shape)
                r = atoms.get_positions()
                if len(r):
                    print("Limits to the positions:")
                    print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                           (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                            min(r[:,2]), max(r[:,2])))
                if world.size > 1:
                    print("Ghost data on processor", i)
                    for key in atoms.ghosts.keys():
                        print("  ", key, atoms.ghosts[key].shape)
                    r = atoms.ghosts['positions']
                    if len(r):
                        print("Limits to the ghost positions:")
                        print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                               (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                                min(r[:,2]), max(r[:,2])))
            world.barrier()


    # Ensure same time step as in Asap-2
    timeunit = 1.018047e-14             # Seconds
    femtosecond = 1e-15 / timeunit      # Marginally different from units.fs

    if isparallel:
        print("RUNNING PARALLEL VERSION OF TEST SCRIPT")
    else:
        print("RUNNING SERIAL VERSION OF TEST SCRIPT")

    if ismaster:
        # First, load a small system
        if os.path.exists("Verlet_legacymass.pickle"):
            data = load(open("Verlet_legacymass.pickle", "rb"), **picklehack)
        else:
            data = load(open("../Verlet_legacymass.pickle", "rb"), **picklehack)
        init_pos = array(data["initial"])
        init_pos.shape = (-1,3)
        init_box = array(data["box"])
        init_box.shape = (3,3)
        a = Atoms(positions=init_pos, cell=init_box, pbc=(1,1,1))
        a.set_atomic_numbers(29*ones(len(a)))
        #atoms = a.repeat((2,2,3))
        # Ensure same order of atoms as in ASE-2
        atoms = a.repeat((1,1,3)).repeat((1,2,1)).repeat((2,1,1))
        dx = 0.1 * sin(arange(3*len(atoms))/10.0)
        dx.shape = (-1,3)
        atoms.set_positions(atoms.get_positions() + dx)
        del dx
        out = sys.stderr
    else:
        atoms = None
        out = open("/dev/null", "w")

    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpulayout)
        nTotalAtoms = atoms.get_number_of_atoms()
    else:
        nTotalAtoms = len(atoms)

    #report()

    print("Setting potential")
    atoms.set_calculator(calc)
    dyn = VelocityVerlet(atoms, 2 * femtosecond)
    # if isparallel:
    #     traj = ParallelNetCDFTrajectory("ptraj.nc", atoms, interval=20)
    # else:
    #     traj = NetCDFTrajectory("ptraj.nc", atoms, interval=20)
    # dyn.Attach(traj)
    # traj.Update()

    print("Number of atoms:", nTotalAtoms)

    epot = atoms.get_potential_energy() / nTotalAtoms
    ekin = atoms.get_kinetic_energy() / nTotalAtoms
    etotallist = [epot+ekin]
    ekinlist = [ekin]

    #report()

    if ismaster:
        print("\nE_pot = %-12.5f  E_kin = %-12.5f  E_tot = %-12.5f" % (epot, ekin,
                                                                     epot+ekin))
    ReportTest("Initial potential energy", epot,-3.4669, 1e-4)
    ReportTest("Initial kinetic energy", ekin, 0.0, 1e-9)

    dyn.attach(MDLogger(dyn, atoms, "-", peratom=True), interval=10)

    for i in range(40):
        dyn.run(5)
        epot = atoms.get_potential_energy() / nTotalAtoms
        ekin = atoms.get_kinetic_energy() / nTotalAtoms
        etotallist.append(epot+ekin)
        ekinlist.append(ekin)

    if ismaster:
        print("Average total energy:", sum(etotallist)/len(etotallist))
        print("Average kinetic energy:", sum(ekinlist)/len(ekinlist))

    ReportTest("Agv. total energy", sum(etotallist)/len(etotallist), -3.4669,
               0.0001)
    ReportTest("Agv. kinetic energy", sum(ekinlist)/len(ekinlist), 0.02165,
               0.002)
    ReportTest.Summary(0)

else:
    print("OpenKIM support is not compiled into Asap.")



"""Asap module Collector.

Defines a filter collecting all information about parallel atoms on the master.
"""

import asap3
import numpy as np

class Collector:
    "Atoms-like filter collecting information on the master node."
    _is_asap_collector_object=True

    def __init__(self, atoms, master=None, allow_forces=False):
        self.atoms = atoms
        self.comm = atoms.get_comm()
        self.allow_forces = allow_forces
        if master is None:
            master = (self.comm.rank == 0)
        self.master = master
        self.constraints = self.atoms.constraints
        self.reset_collector()
        
    def reset_collector(self):
        """Cache some stuff so Trajectory does not block."""
        self._n = self.atoms.get_number_of_atoms()
        try:
            del self.numbers
        except AttributeError:
            pass
        self.numbers = self.get_atomic_numbers()

    def __len__(self):
        if self.master:
            return self._n
        else:
            return 0

    def get_number_of_atoms(self):
        return self.atoms.get_number_of_atoms()

    def get_positions(self):
        return self.collect(self.atoms.get_positions)

    def get_forces(self):
        return self.collect(self.atoms.get_forces)

    def get_momenta(self):
        return self.collect(self.atoms.get_momenta)

    def get_atomic_numbers(self):
        if hasattr(self, "numbers"):
            return self.numbers
        else:
            return self.collect(self.atoms.get_atomic_numbers)
    
    def get_tags(self):
        return self.collect(self.atoms.get_tags)

    def get_potential_energy(self):
        return self.atoms.get_potential_energy()

    def get_cell(self):
        return self.atoms.get_cell()

    def get_calculator(self):
        """Return a fake calculator that makes Trajectory work when it calls calculation_required."""
        return CollectorCalculator(self.atoms.get_calculator(), self.allow_forces)

    def get_stress(self):
        return self.atoms.get_stress()

    def get_pbc(self):
        return self.atoms.get_pbc()
    
    def get_info(self):
        return self.atoms.info
    
    def get_charges(self):
        raise asap3.PropertyNotImplementedError
    
    def get_array(self, label):
        return self.collect(lambda a=self.atoms, l=label: a.get_array(l))

    def has(self, name):
        """Check for existance of array.

        name must be one of: 'tags', 'momenta', 'masses', 'magmoms',
        'charges'.
        """
        if name in ['positions', 'tags', 'momenta', 'numbers']:
            return self.atoms.has(name) 
        else:
            return False

    def iterimages(self):
        """An atom is also a list of images."""
        yield self

    def collect(self, method):
        "Collect data from all cpus onto the master."
        ids = self.atoms.get_ids()
        data = method()
        n = self.atoms.get_number_of_atoms()
        if self.master:
            shape = (n,) + data.shape[1:]
            result = np.zeros(shape, data.dtype)
            for cpu in range(self.comm.size):
                if cpu != 0:
                    # Receive from cpu
                    nrecv = np.zeros(1, int)
                    self.comm.receive(nrecv, cpu)
                    nrecv = nrecv[0]
                    ids = np.zeros(nrecv, int)
                    data = np.zeros((nrecv,) + result.shape[1:], result.dtype)
                    self.comm.receive(ids, cpu)
                    self.comm.receive(data, cpu)
                result[ids] = data
            return result
        else:
            assert(len(data) == len(ids))
            nsend = np.array([len(ids)])
            self.comm.send(nsend, 0)
            self.comm.send(ids, 0)
            self.comm.send(data, 0)
            return np.zeros((0,)+data.shape[1:], dtype=data.dtype)

    def _cant_set_pbc(self, pbc):
        "Fake set_pbc method."
        raise NotImplementedError("Cannot change PBC of a Collector instance.")
    
    pbc = property(get_pbc, _cant_set_pbc, "The boundary conditions attribute")
    
    def _cant_set_numbers(self, z):
        "Fake set_atomic_numbers method."
        raise NotImplementedError(
            "Cannot change atomic numbers of a Collector instance.")
   
    def _cant_set_info(self, info):
        "Cannot set info attribute of Collector instance"
        raise NotImplementedError("Cannot set info attribute of Collector instance")
    
    info = property(get_info, _cant_set_info, "The info dictionary")
    
class CollectorCalculator:
    """Fake calculator object returned by Collector.
    
    This object implements the minimal interface Trajectory needs when it
    tries to write the Collector object as if it were an Atoms object, and
    needs to test which quantities are available.  The calls are forwarded to
    the real potential acting on the real atoms.
    """
    def __init__(self, calc, allow_forces):
        try:
            self.name = calc.name
        except AttributeError:
            try:
                self.name = "asap3." + calc._get_name()
            except AttributeError:
                self.name = calc.__class__.__name__
        self.allow_forces = allow_forces
        
    def calculation_required(self, atoms, props):
        if not self.allow_forces and 'forces' in props:
            return True
        assert isinstance(atoms, Collector)
        realcalc = atoms.atoms.get_calculator()
        x = realcalc.calculation_required(atoms.atoms, props)
        return x
    
    # def get_potential_energy(self, atoms):
    #     return atoms.atoms.get_potential_energy()
    
    # def get_forces(self, atoms):
    #     return atoms.atoms.get_forces()

    def get_property(self, prop, atoms, allow_calculation=True):
        assert isinstance(atoms, Collector)
        try:
            if (not allow_calculation and
                self.calculation_required(atoms, [prop])):
                return None
        except AttributeError:
            pass

        method = 'get_' + {'energy': 'potential_energy'}.get(prop, prop)
        try:
            result = getattr(atoms, method)()
        except AttributeError:
            raise asap3.PropertyNotImplementedError
        return result
    
                     
            



from __future__ import print_function
from asap3 import *
from pickle import *
from numpy import *
from asap3.testtools import ReportTest
from OpenKIM_modelname import openkimmodel
import ase.data

timeunit = 1.018047e-14             # Seconds
femtosecond = 1e-15 / timeunit      # Marginally different from units.fs

print_version(1)

if sys.version_info[0] >= 3:
    picklehack = {'encoding': 'latin1'}
else:
    picklehack = {}

if hasattr(ase.data, 'atomic_masses_legacy'):
    # Newer ASE
    picklefile = "Verlet.pickle"
else:
    picklefile = "Verlet_legacymass.pickle"

if OpenKIMsupported:
    try:
        calc = OpenKIMcalculator(openkimmodel)
    except AsapError as oops:
        if oops.args[0].startswith('Failed to initialize OpenKIM model'):
            print("OpenKIM model {} not installed - skipping test.".format(openkimmodel))
            calc = None
        else:
            raise

    
if OpenKIMsupported and calc is not None:
    data = load(open(picklefile, "rb"), **picklehack)
    init_pos = array(data["initial"])
    init_pos.shape = (-1,3)
    init_box = array(data["box"])
    init_box.shape = (3,3)
    atoms = Atoms(positions=init_pos, cell=init_box)
    atoms.set_atomic_numbers(47*ones((len(atoms),)))
    atoms.set_calculator(calc)
    dyn = VelocityVerlet(atoms, 2 * femtosecond)
    dyn.attach(MDLogger(dyn, atoms, '-', peratom=True), interval=5)
    
    etot = None
    
    for i in range(10):
        dyn.run(20)
        epot = atoms.get_potential_energy() / len(atoms)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        if etot is None:
            etot = epot + ekin
        else:
            ReportTest("Energy conservation:", epot + ekin, etot, 0.001)
            
    final_pos = array(data["final"])
    diff = max(abs(atoms.get_positions().flat - final_pos))
    print("Maximal deviation of positions:", diff)
    ReportTest("Maximal deviation of positions", diff, 0, 1e-9)
    
    #diff = max(abs(atoms.get_stresses().flat - array(data["stress"])))
    #print "Maximal deviation of stresses:", diff
    #ReportTest("Maximal deviation of stresses", diff, 0, 1e-9)
    
    ReportTest.Summary()

else:
    print("OpenKIM support is not compiled into Asap.")
    



from __future__ import print_function
from ase.structure import molecule
from ase.structure import nanotube
from ase.optimize import BFGS, MDMin
from asap3 import BrennerPotential
from asap3.testtools import ReportTest
import numpy as np

atoms = nanotube(5, 5, length=10)
atoms.center(axis=(0,1), vacuum=0.0)
atoms.set_calculator(BrennerPotential())
uc = atoms.get_cell()
l0 = uc[2,2]

e = -1e100
for eps in np.linspace(0, 0.5, 201):
    uc[2,2] = l0 * (1 + eps)
    atoms.set_cell(uc, scale_atoms=True)
    dyn = MDMin(atoms, logfile=None, dt=0.05)
    dyn.run(fmax=0.01, steps=100)
    epot = atoms.get_potential_energy()
    if epot > e:
        e = epot
    print("%.3f  %f" % (eps, epot))

print("Maximal energy:", e)
ReportTest("Maximal energy", e, -969.0, 5.0)
if abs(e - -620.98) < 1.0:
    print("Bug 42 is back!")



#!/usr/bin/env python

"""This is the setup and installation script for Asap.

To install, simply run
    python setup.py install --user

CUSTOMIZATION:
If you need to change compiler options or other stuff, 
do NOT EDIT THIS FILE.  Instead, you should edit 
customize.py or one of its friends.

Customization information is read from the first of
these files found:
    customize-local.py
    customize-hostname.domain.py
    customize.py
If setup.py is given the option --customize=myfile.py then
that file is used instead of the above.

Only one of these files is read.  See customize.py for further
instructions.  But remember: MOST USERS DO NOT NEED THIS!

"""

from __future__ import print_function
import distutils
import distutils.util
import distutils.spawn
import os
import os.path as op
import stat
import re
import sys
from distutils.command.build_ext import build_ext as _build_ext
from distutils.command.build_scripts import build_scripts as _build_scripts
#from distutils.command.sdist import sdist as _sdist
from distutils.core import setup, Extension
from distutils.sysconfig import get_config_vars
from glob import glob
import numpy
import platform
import subprocess
import recordversion   # In this project - records the compiler information into CPP

assert sys.version_info >= (2, 6)

description = "ASAP - classical potentials for MD with ASE."

long_description = """\
ASAP (Atomic SimulAtion Program or As Soon As Possible) is a
package for large-scale molecular dynamics within the Atomic
Simulation Environment (ASE).  It implements a number of 'classical'
potentials, most importantly the Effective Medium Theory, and also the
mechanisms for domain-decomposition of the atoms."""

folders = ['Basics', 'Potentials', 'Interface', 'Brenner', 'Tools',
           'PTM', 'PTM/qcprot', 'PTM/voronoi']
kim_folders = ['OpenKIMimport']
parallel_folders = ['Parallel', 'ParallelInterface']
exclude_files = ['Interface/AsapModule.cpp']
serial_only_files = ['Interface/AsapSerial.cpp']


libraries = ['m']
library_dirs = []
include_dirs = []
extra_link_args = []
extra_compile_args = []
remove_compile_args = []
runtime_library_dirs = []
extra_objects = []
define_macros = [] 
undef_macros = ['NDEBUG']

mpi_libraries = []
mpi_library_dirs = []
mpi_include_dirs = []
mpi_runtime_library_dirs = []
mpi_define_macros = []
mpi_undef_macros = []
mpi_compiler = ('mpicc', 'mpicxx')

use_intel_compiler = False  # False, True or 'auto' - enable in customize.py
intel_compiler = ('icc', 'icpc')
intel_mpi_compiler = ('env OMPI_MPICC=icc mpicc', 'env OMPI_MPICXX=icpc mpicxx')
intel_compile_args = ['-O3', '-g', '-xHost', '-fPIC']
intel_libraries = ['svml', 'm']
intel_link_args = []
# Some Intel compilers are broken, and cause Asap to fail.
intel_blacklist = ['15.0.1', '17.0.1', '17.0.2']

# Default values for compilation with MKL - only if MKL sets $MKLROOT or if
# defined manually in customize.py
mkl_supported = 'MKLROOT' in os.environ
mkl_define_macros = [('ASAP_MKL',1)]
mkl_compile_args = []
mkl_intel_compile_args = []
mkl_libraries = ['mkl_rt']
mkl_library_dirs = []
if mkl_supported:
    mkl_include_dirs = ['{MKLROOT}/include'.format(**os.environ)]
else:
    mkl_include_dirs = []
    
systemname, hostname, dummy1, dummy2, machinename, processorname = platform.uname()

#platform_id = ''  # Not changable for Asap (see gpaw/setup.py)

# Get the version number from Python/asap3/version.py
# Get the current version number:
with open('Python/asap3/version.py') as fd:
    version = re.search('__version__ = "(.*)"', fd.read()).group(1)
print("Asap version number:", version)

# Look for --customize arg
for i, arg in enumerate(sys.argv):
    if arg.startswith('--customize'):
        customize = sys.argv.pop(i).split('=')[1]
        break
else:
    # No --customize arg.
    for c in ('customize-local.py', 
              'customize-{0}.py'.format(hostname),
              'customize.py'):
        if os.path.exists(c):
            customize = c
            break
        elif '-' in c:
            print(c, "not found - this is OK.")
    else:
        raise RuntimeError("customize.py not found - broken source distribution.")

print("Reading customization from", customize)
exec(open(customize).read())

# Check if command line arguments override the use of Intel compilers.
for i, arg in enumerate(sys.argv):
    if arg.lower() == '--with-intel':
        use_intel_compiler = True
        del sys.argv[i]
        break
    if arg.lower() == '--without-intel':
        use_intel_compiler = False
        del sys.argv[i]
        break

# Check if Intel compiler should be used iff present
if use_intel_compiler == 'auto':
    use_intel_compiler = bool(distutils.spawn.find_executable(intel_compiler[0]))
    print("Intel compiler autodetection: use Intel compiler =", use_intel_compiler)

# Check for command line argument asking for MKI
use_mkl = False
for i, arg in enumerate(sys.argv):
    if arg.lower() == '--with-mkl':
        print("Intel Math Kernel Library support enabled.")
        use_mkl = True
        del sys.argv[i]
        break

if use_mkl:
    if not mkl_supported:
        raise RuntimeError("You asked for Intel MKL support, but it is not available. Configure in customize.py or set $MKLROOT")
    extra_compile_args += mkl_compile_args
    intel_compile_args += mkl_intel_compile_args
    include_dirs += mkl_include_dirs
    libraries += mkl_libraries
    library_dirs += mkl_library_dirs
    define_macros += mkl_define_macros

# Check for the --clear-compiler-environment command line option
envirhack = False
for i, arg in enumerate(sys.argv):
    if arg.lower() == '--clear-compiler-environment':
        envirhack = True
        del sys.argv[i]
        break

if envirhack:
    print("Clearing environment variables (e.g. for EasyBuild).")
    for evar in ('CC', 'CXX', 'CFLAGS', 'CXXFLAGS', 'LDFLAGS'):
        if evar in os.environ:
            print("   Clearing {0}  (was '{1}')".format(evar,
                                                        os.environ[evar]))
            del os.environ[evar]
# Check if the MPI compiler is present.

if mpi_compiler:
    for mpicomp in mpi_compiler:
        if not distutils.spawn.find_executable(mpicomp):
            print("WARNING: No MPI compiler '{}': Not building parallel version.".format(mpicomp))
            mpi_compiler = None
            break

# If intel compiler selected, we need to change some compiler flags.
# This comes after the test for the MPI compiler, so the MPI compiler
# wrapper script is detected properly.
cfgDict = get_config_vars()
if use_intel_compiler:
    print("Using Intel compiler:")
    for var in ['BASECFLAGS', 'CFLAGS', 'OPT', 'PY_CFLAGS',
                'CCSHARED']: # 'CFLAGSFORSHARED', 'LINKFORSHARED']
        cfgDict[var] = ''  # Clear it
    cfgDict['BASECFLAGS'] = cfgDict['CFLAGS'] = ' '.join(intel_compile_args)
    libraries += intel_libraries
    for key in ['CC', 'CXX', 'LDSHARED', 'LDCXXSHARED']:
        if key in cfgDict:
            value = cfgDict[key].split()
            # first argument is the compiler/linker.
            value[0] = intel_compiler[1]  # C++ compiler
            cfgDict[key] = ' '.join(value)
            print("    Setting {0}={1}".format(key, cfgDict[key]))
    mpi_compiler = intel_mpi_compiler
    extra_compile_args = []  # Specified in intel_compile_args
    extra_link_args = intel_link_args
    # Check the version number of the intel compiler
    iccver = subprocess.check_output(intel_compiler[0] + ' -dumpversion', shell=True)
    if iccver in intel_blacklist:
        raise RuntimeError("Cannot compile with Intel compiler version {} due to a compiler bug.".format(iccver))


# Remove unneeded compiler arguments
if remove_compile_args:
    for fvar in ['BASECFLAGS', 'CFLAGS', 'OPT', 'PY_CFLAGS',
                  'CFLAGSFORSHARED']:
        if fvar in cfgDict:
            cflags = cfgDict[fvar].split()
            for arg in remove_compile_args:
                if arg in cflags:
                    cflags.remove(arg)
                    print("Removed compiler argument", arg)
            cfgDict[fvar] = ' '.join(cflags)
    
def runcmd(cmd, verb=False):
    if verb:
        print(cmd)
    x = os.system(cmd)
    if x:
        raise RuntimeError("Command failed: "+cmd)

def build_interpreter(compiler, common_src, parallel_src,
                      libraries, library_dirs, include_dirs, 
                      extra_link_args, extra_compile_args,
                      runtime_library_dirs, extra_objects, define_macros,
                      undef_macros):
    """Build the asap-python executable."""
    print("\n\n*** Building the asap-python executable ***\n\n")
    (c_compiler, cpp_compiler) = compiler

    # Get configuration variable
    cfgDict = get_config_vars()
    plat = distutils.util.get_platform() + '-' + sys.version[0:3]
    if hasattr(sys, 'gettotalrefcount'):
        plat += '-pydebug'

    # Split sources into C and C++, and find object names.
    c_sources = {}
    cpp_sources = {}
    common_sources = {} # Will not be compiled, don't care about C/C++
    for s in parallel_src:
        dir = 'build/temp.{0}/{1}'.format(plat, os.path.dirname(s))
        if not os.path.exists(dir):
            os.mkdir(dir)   # Create folder for object file.
        if s.endswith('.cpp'):
            objname = 'build/temp.{0}/{1}.o'.format(plat, s[:-4])
            cpp_sources[s] = objname
        elif s.endswith('.c'):
            objname = 'build/temp.{0}/{1}.o'.format(plat, s[:-2])
            c_sources[s] = objname
        else:
            raise RuntimeError("Source file {0} is neither C nor C++.".format(s))
    for s in common_src:
        if s.endswith('.cpp'):
            objname = 'build/temp.{0}/{1}.o'.format(plat, s[:-4])
            common_sources[s] = objname
        elif s.endswith('.c'):
            objname = 'build/temp.{0}/{1}.o'.format(plat, s[:-2])
            common_sources[s] = objname
        else:
            raise RuntimeError("Source file {0} is neither C nor C++.".format(s))        
    exefile = "build/bin.{0}/asap-python".format(plat)
    if not os.path.exists("build/bin.{0}".format(plat)):
        os.mkdir("build/bin.{0}".format(plat))
        
    # Now construct the compilation command lines
    macros = ' '.join(['-D%s=%s' % x for x in define_macros if x[0].strip()])
    macros += ' '.join(['-U%s' % (x,) for x in undef_macros if x.strip()])
    
    include_dirs = include_dirs + [cfgDict['INCLUDEPY'],
                                   cfgDict['CONFINCLUDEPY']]
    includes = ' '.join(['-I' + incdir for incdir in include_dirs])

    library_dirs = library_dirs + [cfgDict['LIBPL']]
    lib_dirs = ' '.join(['-L' + lib for lib in library_dirs])

    libs = ' '.join(['-l' + lib for lib in libraries if lib.strip()])
    libs += ' ' + cfgDict.get('BLDLIBRARY', '-lpython%s' % cfgDict['VERSION'])
    libs += ' ' + ' '.join([libs, cfgDict['LIBS'], cfgDict['LIBM']])
    if sys.platform == 'darwin':
        runtime_libs = ''   # No -rpath on macOS
    else:
        runtime_lib_option = '-Wl,-rpath='
        runtime_libs = ' '.join([runtime_lib_option + lib
                                for lib in runtime_library_dirs])
    extra_link_args = extra_link_args + [cfgDict['LDFLAGS'],
                                         cfgDict['MODLIBS'],
                                         cfgDict['LINKFORSHARED']]
    # Compile
    all_objects = []
    cflags = cfgDict['CFLAGS'].split()
    compile_args_c = cflags + list(extra_compile_args)
    compile_args_cpp = cflags + list(extra_compile_args)
    if '-std=c++11' in compile_args_c:
        compile_args_c.remove('-std=c++11')
    for compiler, sources, compargs in (\
                (c_compiler, c_sources, compile_args_c),
                                        (cpp_compiler, cpp_sources, compile_args_cpp)):
        print("*** COMPILING: ", list(sorted(sources.keys())))
        for src in sorted(sources.keys()):
            obj = sources[src]
            all_objects.append(obj)
            cmd = ('%s %s %s %s -o %s -c %s ') % \
                  (compiler,
                   ' '.join(compargs),
                   macros,
                   includes,
                   obj,
                   src)
            runcmd(cmd, True)
    # Link
    all_objects = list(common_sources.values()) + all_objects
    cmd = ('%s -o %s %s %s %s %s %s %s') % \
          (cpp_compiler,
           exefile,
           ' '.join(all_objects),
           ' '.join(extra_objects),
           lib_dirs,
           libs,
           runtime_libs,
           ' '.join(extra_link_args))
    runcmd(cmd, True)


# Check if we are making a source distribution.  In that case, we
# should remove the VersionInfo_autogen folder and all files in it.
versiondir = "VersionInfo_autogen"
is_making_distro = 'sdist' in sys.argv
if is_making_distro and os.path.isdir(versiondir):
    print("Clearing", versiondir)
    for f in os.listdir(versiondir):
        os.remove(os.path.join(versiondir, f))
    os.rmdir(versiondir)

# Create the version.cpp file
try:
    host = os.uname()[1]
except:
    host = 'unknown'
if not is_making_distro:
    versioninfo_s = "{0}/version_{1}_s.cpp".format(versiondir, host)
    versioninfo_p = "{0}/version_{1}_p.cpp".format(versiondir, host)
    if not os.path.exists(versiondir):
        os.mkdir("VersionInfo_autogen")
    t = os.stat("Python/asap3/version.py")[stat.ST_MTIME]
    if (not os.path.exists(versioninfo_s) or t > os.stat(versioninfo_s)[stat.ST_MTIME] or
        not os.path.exists(versioninfo_p) or t > os.stat(versioninfo_p)[stat.ST_MTIME]):
        print("Recording version info into VersionInfo_autogen")
        print("  CC =", cfgDict['CC'])
        print("  CFLAGS =", cfgDict['CFLAGS'])
        serialcomp = "distutils with {0} {1}".format(cfgDict['CC'], cfgDict['CFLAGS'])
        with open(versioninfo_s, "wt") as versioncpp:
            versioncpp.write(recordversion.contents % (version, 'serial', host, serialcomp, version))
        if mpi_compiler is not None:
            parallelcomp = "distutils with {0} {1}".format(mpi_compiler[1], 
                                                           " ".join([cfgDict['CFLAGS']]
                                                                    +extra_compile_args))
            with open(versioninfo_p, "wt") as versioncpp:
                versioncpp.write(recordversion.contents % (version, 'parallel', host, parallelcomp, version))


if kim_folders and 'KIM_HOME' in os.environ:
    kh = os.environ['KIM_HOME']
    include_dirs += [os.path.join(kh, 'include', 'kim-api')]
    include_dirs += kim_folders
    library_dirs += [os.path.join(kh, 'lib64'), os.path.join(kh, 'lib'),]
    runtime_library_dirs += [os.path.join(kh, 'lib64'), os.path.join(kh, 'lib')]
    libraries += ['kim-api']
    define_macros += [('WITH_OPENKIM', '1')]
    folders.extend(kim_folders)
else:
    # Try to use pkgconfig to locate OpenKIM
    failed = subprocess.call("pkg-config --exists libkim-api",
                             shell=True)
    if not failed:
        # pkg-config is installed and so is OpenKIM.
        kimincl = subprocess.check_output(
            "pkg-config --cflags-only-I libkim-api", shell=True).decode()
        include_dirs += [f.strip() for f in kimincl.split('-I') if f]
        include_dirs += kim_folders
        kimlibd = subprocess.check_output(
            "pkg-config --libs-only-L libkim-api", shell=True).decode()
        kimliblist = [f.strip() for f in kimlibd.split('-L') if f]
        library_dirs += kimliblist
        runtime_library_dirs += kimliblist
        kimlibl = subprocess.check_output(
            "pkg-config --libs-only-l libkim-api", shell=True).decode()
        libraries += [f.strip() for f in kimlibl.split('-l') if f]
        define_macros += [('WITH_OPENKIM', '1')]
        folders.extend(kim_folders)
        
include_dirs += folders
include_dirs.append(numpy.get_include())
mpi_include_dirs += parallel_folders

print("Identifying source files")
# Find source files for serial compilation
common_src_files = []
for d in folders:
    for f in os.listdir(d):
        if f.endswith('.cpp'):
            fn = os.path.join(d,f)
            if fn not in (exclude_files + serial_only_files):
                common_src_files.append(fn)
                #print("  ", fn)
serial_src_files = common_src_files + serial_only_files
if not is_making_distro:
    serial_src_files.append(versioninfo_s)
serial_src_files.sort()
parallel_src_files = []
for d in parallel_folders:
    for f in os.listdir(d):
        if f.endswith('.cpp') or f.endswith('.c'):
            fn = os.path.join(d,f)
            parallel_src_files.append(fn)
if not is_making_distro:
    parallel_src_files.append(versioninfo_p)

print("Identifying Python submodules")
packages = []
for dirname, dirnames, filenames in os.walk('Python/asap3'):
    if '__init__.py' in filenames:
        dname = dirname.split('/')[1:]  # Remove leading Python/
        packages.append('.'.join(dname))

extensions = [Extension('_asap_p{0}'.format(sys.version_info[0]),
                        serial_src_files,
                        libraries=libraries,
                        library_dirs=library_dirs,
                        include_dirs=include_dirs,
                        define_macros=define_macros,
                        undef_macros=undef_macros,
                        extra_link_args=extra_link_args,
                        extra_compile_args=extra_compile_args,
                        runtime_library_dirs=runtime_library_dirs,
                        )]

# Scripts
scripts = ['scripts/asap-qsub', 'scripts/asap-sbatch']
if mpi_compiler:
    plat = distutils.util.get_platform() + '-' + sys.version[0:3]
    if hasattr(sys, 'gettotalrefcount'):
        plat += '-pydebug'
    scripts.append("build/bin.{0}/asap-python".format(plat))
    
class build_ext(_build_ext):
    def run(self):
        _build_ext.run(self)
        if mpi_compiler:
            # Also build asap-python:
            build_interpreter(
                mpi_compiler,
                common_src_files,
                parallel_src_files,
                libraries=libraries+mpi_libraries,
                library_dirs=library_dirs+mpi_library_dirs,
                include_dirs=include_dirs+mpi_include_dirs,
                extra_link_args=extra_link_args,
                extra_compile_args=extra_compile_args,
                runtime_library_dirs=runtime_library_dirs,
                extra_objects=extra_objects,
                define_macros=define_macros+mpi_define_macros,
                undef_macros=undef_macros+mpi_undef_macros)

class build_scripts(_build_scripts):
    # Python 3 will try to read the asap-python executable, handle
    # that one specially.
    def copy_scripts(self):
        executables = []
        for i, s in enumerate(self.scripts):
            if 'asap-python' in s:
                print("Protecting", s, "from ajustment")
                executables.append(s)
        for s in executables:
            self.scripts.remove(s)
        assert len(executables) <= 1
        x =_build_scripts.copy_scripts(self)
        if x is None:
            # Python 2
            outfiles = []
            updated_files = []
        else:
            outfiles, updated_files = x
        for script in executables:
            outfile = op.join(self.build_dir, op.basename(script))
            outfiles.append(outfile)
            updated_files.append(outfile)
            self.copy_file(script, outfile)
        return outfiles, updated_files
            
setup(name="asap3",
      version=version,
      description=description,
      long_description=long_description,
      maintainer="Jakob Schiotz et. al.",
      maintainer_email="schiotz@fysik.dtu.dk",
      url="https://wiki.fysik.dtu.dk/asap",
      packages=packages,
      package_dir={'asap3': 'Python/asap3'},
      ext_modules=extensions,
      license='LGPLv3',
      platforms=['unix'],
      scripts=scripts,
      cmdclass={'build_ext': build_ext,
                'build_scripts': build_scripts},
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
          'Operating System :: POSIX',
          'Operating System :: MacOS :: MacOS X',
                    'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Topic :: Scientific/Engineering :: Physics'
      ])




import sys
import time
import numpy as np
from ase.utils import basestring

from ase.io import Trajectory

from asap3.Tools.Timing import Timer

class GlobalOptimizer:
    def __init__(self, atoms, log='-', traj=None):
        self.atoms = atoms
        self.move = None
        self.moves = []
        self.moves_weight = []
        self.observers = []
        self.nsteps = {}
        self.naccept = {}
        self.timer = Timer()

        self.log_init = ''
        self.log_header = ''
        self.log_string = ''
        self.log_final = ''

        if isinstance(traj, basestring):
            traj = Trajectory(traj, 'w', atoms, master=True)
            traj.write()
            self.attach_observer(traj.write)

        if isinstance(log, basestring):
            if log == '-':
                self.logfile = sys.stdout
            else:
                self.logfile = open(log, 'a')
        else:
            self.logfile = None

    def attach_move(self, move, weight=1.0):
        if not hasattr(move, '__call__'):
            raise ValueError("Attached move is not callable.")

        if hasattr(move, 'set_atoms'):
            move.set_atoms(self.atoms)

        if hasattr(move, 'set_optimizer'):
            move.set_optimizer(self)

        self.moves.append(move)
        self.moves_weight.append(weight)

    def random_move(self):
        return self.moves[(np.random.uniform() < self.moves_weight).argmax()]

    def attach_observer(self, observer, interval=1, attime='accept', *args, **kwargs):
        if not hasattr(observer, '__call__'):
            raise ValueError("Attached observer is not callable.")

        if hasattr(observer, 'set_atoms'):
            observer.set_atoms(self.atoms)

        if hasattr(observer, 'set_optimizer'):
            observer.set_optimizer(self)

        self.observers.append((observer, interval, attime, args, kwargs))

    def call_observers(self, thistime):
        for observer, interval, attime, args, kwargs in self.observers:
            if self.get_nsteps() % interval == 0 and attime == thistime:
                observer(*args, **kwargs)

    def initialize(self):
        if not len(self.moves) > 0:
            raise ValueError("You must attach at least one move.")

        w = np.array(self.moves_weight, dtype=float)
        self.moves_weight = np.add.accumulate(w/w.sum())

        for m in self.moves:
            self.nsteps[m.get_name()] = 0
            self.naccept[m.get_name()] = 0

    def finalize(self):
        pass

    def run(self, steps, *args, **kwargs):
        self.timer.start('Initialize')
        self.initialize(*args, **kwargs)
        self.timer.stop('Initialize')

        if self.logfile is not None:
            name = self.__class__.__name__
            T = time.localtime()
            self.logfile.write("\n\n%s global optimization, " % (name,) +
                               "%02i-%02i-%i\n" % (T[2], T[1], T[0]) +
                               "*" * 75 + "\n" +
                               "%-15s%-7s\n" % ("Move", "Weight"))

            for m, w in zip(self.moves, self.moves_weight):
                self.logfile.write("%-15s%-7.5f\n" % (m.get_name(), w))

            self.logfile.write(self.log_init)
            self.logfile.write('\n%-8s  %-7s  %-11s  ' %
                               ('Time', 'Steps', 'Move') + self.log_header)
            self.logfile.flush()

        for i in range(steps):
            self.timer.start('Move')
            self.move = self.random_move()
            self.move(self.atoms)
            self.timer.stop('Move')

            self.call_observers('move')

            self.timer.start('Evaluate')
            accept = self.evaluate_move()
            self.timer.stop('Evaluate')

            self.call_observers('evaluate')

            if accept:
                self.accept_move()
            else:
                self.reject_move()

            self.call_observers('after')

            if self.logfile is not None:
                name = self.move.get_name()
                T = time.localtime()
                self.logfile.write('%02d:%02d:%02d  %-7i  %-11s  ' %
                                   (T[3], T[4], T[5], self.get_nsteps(),
                                    name[:-4]) + self.log_string)
                self.logfile.flush()

        self.finalize()

        if self.logfile is not None:
            self.logfile.write('\n%-13s  %-15s  %-15s\n' %
                               ('Move', 'Steps', 'Accepts'))
            self.logfile.write('=' * 60 + '\n')
            for m in self.moves:
                name = m.get_name()
                ns = self.get_nsteps(name)
                fs = 1.0 * ns / self.get_nsteps()
                na = self.get_naccept(name)
                fa = 1.0 * na / ns
                self.logfile.write('%-13s  %-7i (%5.3f)  %-7i (%5.3f)\n' %
                                   (name, ns, fs, na, fa))
            self.logfile.write('=' * 60 + '\n')
            ns = self.get_nsteps()
            na = self.get_naccept()
            self.logfile.write('%-13s  %-7i (%5.3f)  %-7i (%5.3f)\n' %
                               ('Total', ns, 1.0, na, 1.0 * na / ns))
            self.logfile.write('=' * 60 + '\n')
            self.logfile.write(self.log_final)
            self.timer.write(self.logfile)
            self.logfile.flush()

    def evaluate_move(self):
        return True

    def accept_move(self):
        self.move.accept()
        self.call_observers('accept')
        self.naccept[self.move.get_name()] += 1
        self.nsteps[self.move.get_name()] += 1

    def reject_move(self):
        self.move.reject()
        self.call_observers('reject')
        self.nsteps[self.move.get_name()] += 1

    def get_nsteps(self, name='total'):
        if name == 'total':
            return sum(self.nsteps.values())
        else:
            return self.nsteps[name]

    def get_naccept(self, name='total'):
        if name == 'total':
            return sum(self.naccept.values())
        else:
            return self.naccept[name]




import numpy as np

from asap3.MonteCarlo.Base import GlobalOptimizer
from ase.units import kB

class Metropolis(GlobalOptimizer):
    def __init__(self, atoms, log='-', traj=None):
        GlobalOptimizer.__init__(self, atoms, log, traj)
        self.Eo = atoms.get_potential_energy()
        self.En = 1.0e32
        self.P = 0.0

        self.log_header = ('%-15s  %-12s  %-6s\n' %
                           ('Energy', 'Probability', 'Accept'))

    def initialize(self, temp=1000.0):
        GlobalOptimizer.initialize(self)
        self.temp = temp
        self.log_init = '\nTemperature: %i\n' % (temp,)

    def evaluate_move(self):
        self.En = self.atoms.get_potential_energy()

        self.P = min(1.0, np.exp((self.Eo - self.En) / (self.temp * kB)))
        accept = self.P > np.random.uniform()

        self.log_string = ('%-15.6f  %-12.6f  %-6s\n' %
                           (self.En, self.P, accept))

        return accept

    def accept_move(self):
        self.Eo = self.En
        GlobalOptimizer.accept_move(self)

